#include <wx/wxprec.h>
#include <wx/wxprec.h>

#ifdef __BORLANDC__
        #pragma hdrstop
#endif

#ifndef WX_PRECOMP
        #include <wx/wx.h>
#endif

#include <wx/filename.h>
#include "OggFile.h"
#include "OggMerger.h"
#include "ProgressDialog.h"
#include "gh2makerMain.h"
#include "FinalizerThread.h"
#include "MidiFile.h"

#ifdef __cplusplus
extern "C" {
#endif

#include "dtb/dtb.h"
int midicomp_decompile( char *ifilename, char *ofilename );

#ifdef __cplusplus
}
#endif


wxCriticalSection g_crit ;
wxCriticalSection g_crit_grid ;

extern wxSemaphore g_semProgress;
extern wxSemaphore g_semDialog;
extern gh2makerFrame *g_frame ;
extern int g_num_converted_ogg ;
extern int g_dialog_answer ;
FinalizerThread *g_finalizer ;

extern unsigned char *dummy_mid_bytes ;
extern unsigned char *dummy_ogg_bytes ;


void FinalizerThread_UpdateProgress( char *str1, char *str2, int currnum )
{
    g_finalizer->OverallProgress( g_finalizer->m_current_step++, g_finalizer->m_total_steps, wxString::Format( _T("Injecting %d of %d ARK files"), currnum, g_finalizer->m_num_injections  ) )  ;
    g_finalizer->CurrentProgress( currnum, g_finalizer->m_num_injections, wxString::Format( _T("Injecting %s"), wxString::FromAscii(str2).c_str() ) )  ;

}

void FinalizerThread::OnExit()
{
    g_semProgress.Post() ;
}


void FinalizerThread::Error( wxString msg )
{

    m_done = true ;

    wxCommandEvent event( wxEVT_FINISHED_EVENT, m_notify_id  );
    event.SetString( msg ) ;
    event.SetInt( 1 ) ;
    g_crit.Enter() ;
    if ( m_notify_dlg->m_done )
    {
        g_crit.Leave() ;
        return  ;
    }
    wxPostEvent( m_notify_dlg, event );
    wxMutexGuiEnter();
    g_frame->Grid1->Refresh() ;
    wxMutexGuiLeave() ;
    g_crit.Leave() ;

}

void FinalizerThread::PostDialog( wxString msg )
{
    wxCommandEvent event( wxEVT_DIALOG_EVENT, m_notify_id  );
    event.SetString( msg ) ;
    g_crit.Enter() ;
    if ( m_notify_dlg->m_done )
    {
        g_crit.Leave() ;
        return  ;
    }
    wxPostEvent( m_notify_dlg, event );
    g_crit.Leave() ;


    g_semDialog.Wait() ;
}

void FinalizerThread::CurrentProgress( int curr, int total, wxString msg )
{
    wxCommandEvent event( wxEVT_CURRENT_PROGRESS_EVENT, m_notify_id  );
    if ( total )
        event.SetInt( (curr*100)/total ) ;
    else
        event.SetInt( 0 ) ;
    event.SetString( msg ) ;
    g_crit.Enter() ;
    if ( m_notify_dlg->m_done )
    {
        g_crit.Leave() ;
        return  ;
    }
    wxPostEvent( m_notify_dlg, event );
    g_crit.Leave() ;

}

void FinalizerThread::OverallProgress( int curr, int total, wxString msg )
{
    wxCommandEvent event( wxEVT_OVERALL_PROGRESS_EVENT, m_notify_id  );
    event.SetInt( (curr*100)/total ) ;
    event.SetString( wxString::Format( _T("Step %d of %d - %s"), curr+1, total, msg.c_str() ) ) ;
    g_crit.Enter() ;
    if ( m_notify_dlg->m_done )
    {
        g_crit.Leave() ;
        return  ;
    }
    wxPostEvent( m_notify_dlg, event );
    g_crit.Leave() ;
}


typedef struct
{
    char *mid ;
    char *voc ;
    char *single ;
    char *coop ;
    char *trackid ;
    char *trackid_coop ;
    char *vols ;
    char *vols_coop ;
    char *pans ;
    char *pans_coop ;
    char *cores ;
    char *cores_coop ;

} song_filenames_struct ;

song_filenames_struct song_filenames[] =
{
    {
        "songs/surrender/surrender.mid",
        "songs/surrender/surrender.voc",
        "songs/surrender/surrender.mogg",
        "",
        "36.3.2.0",
        "",
        "36.3.4.0",
        "",
        "36.3.3.0",
        "",
        "36.3.5.0",
        "",
    },
    {
        "songs/possum/possum.mid",
        "songs/possum/possum.voc",
        "songs/possum/possum.mogg",
        "",
        "27.4.2.0",
        "",
        "27.4.4.0",
        "",
        "27.4.3.0",
        "",
        "27.4.5.0",
        "",
    },
    {
        "songs/heartshapedbox/heartshapedbox.mid",
        "songs/heartshapedbox/heartshapedbox.voc",
        "songs/heartshapedbox/heartshapedbox.mogg",
        "",
        "13.3.2.0",
        "",
        "13.3.4.0",
        "",
        "13.3.3.0",
        "",
        "13.3.5.0",
        "",
    },
    {
        "songs/salvation/salvation.mid",
        "songs/salvation/salvation.voc",
        "songs/salvation/salvation.mogg",
        "",
        "31.3.2.0",
        "",
        "31.3.4.0",
        "",
        "31.3.3.0",
        "",
        "31.3.5.0",
        "",
    },
    {
        "songs/strutter/strutter.mid",
        "songs/strutter/strutter.voc",
        "songs/strutter/strutter.mogg",
        "songs/strutter/strutter_coop.mogg",
        "35.3.2.0",
        "35.4.2.0",
        "35.3.4.0",
        "35.4.4.0",
        "35.3.3.0",
        "35.4.3.0",
        "35.3.5.0",
        "35.4.5.0",
    },
    {
        "songs/shoutatthedevil/shoutatthedevil.mid",
        "songs/shoutatthedevil/shoutatthedevil.voc",
        "songs/shoutatthedevil/shoutatthedevil.mogg",
        "",
        "33.3.2.0",
        "",
        "33.3.4.0",
        "",
        "33.3.3.0",
        "",
        "33.3.5.0",
        "",
    },
    {
        "songs/mother/mother.mid",
        "songs/mother/mother.voc",
        "songs/mother/mother.mogg",
        "songs/mother/mother_coop.mogg",
        "25.3.2.0",
        "25.4.2.0",
        "25.3.4.0",
        "25.4.4.0",
        "25.3.3.0",
        "25.4.3.0",
        "25.3.5.0",
        "25.4.5.0",
    },
    {
        "songs/lifewasted/lifewasted.mid",
        "songs/lifewasted/lifewasted.voc",
        "songs/lifewasted/lifewasted.mogg",
        "",
        "21.3.2.0",
        "",
        "21.3.4.0",
        "",
        "21.3.3.0",
        "",
        "21.3.5.0",
        "",
    },
    {
        "songs/cherrypie/cherrypie.mid",
        "songs/cherrypie/cherrypie.voc",
        "songs/cherrypie/cherrypie.mogg",
        "",
        "6.3.2.0",
        "",
        "6.3.4.0",
        "",
        "6.3.3.0",
        "",
        "6.3.5.0",
        "",
    },
    {
        "songs/woman/woman.mid",
        "songs/woman/woman.voc",
        "songs/woman/woman.mogg",
        "",
        "45.3.2.0",
        "",
        "45.3.4.0",
        "",
        "45.3.3.0",
        "",
        "45.3.5.0",
        "",
    },
    {
        "songs/youreallygotme/youreallygotme.mid",
        "songs/youreallygotme/youreallygotme.voc",
        "songs/youreallygotme/youreallygotme.mogg",
        "",
        "46.3.2.0",
        "",
        "46.3.4.0",
        "",
        "46.3.3.0",
        "",
        "46.3.5.0",
        "",
    },
    {
        "songs/tonightimgonna/tonightimgonna.mid",
        "songs/tonightimgonna/tonightimgonna.voc",
        "songs/tonightimgonna/tonightimgonna.mogg",
        "songs/tonightimgonna/tonightimgonna_coop.mogg",
        "40.3.2.0",
        "40.4.2.0",
        "40.3.4.0",
        "40.4.4.0",
        "40.3.3.0",
        "40.4.3.0",
        "40.3.5.0",
        "40.4.5.0",
    },
    {
        "songs/carryonwayward/carryonwayward.mid",
        "songs/carryonwayward/carryonwayward.voc",
        "songs/carryonwayward/carryonwayward.mogg",
        "",
        "5.3.2.0",
        "",
        "5.3.4.0",
        "",
        "5.3.3.0",
        "",
        "5.3.5.0",
        "",
    },
    {
        "songs/searchanddestroy/searchanddestroy.mid",
        "songs/searchanddestroy/searchanddestroy.voc",
        "songs/searchanddestroy/searchanddestroy.mogg",
        "",
        "32.3.2.0",
        "",
        "32.3.4.0",
        "",
        "32.3.3.0",
        "",
        "32.3.5.0",
        "",
    },
    {
        "songs/messageinabottle/messageinabottle.mid",
        "songs/messageinabottle/messageinabottle.voc",
        "songs/messageinabottle/messageinabottle.mogg",
        "",
        "23.3.2.0",
        "",
        "23.3.4.0",
        "",
        "23.3.3.0",
        "",
        "23.3.5.0",
        "",
    },
    {
        "songs/billiondollar/billiondollar.mid",
        "songs/billiondollar/billiondollar.voc",
        "songs/billiondollar/billiondollar.mogg",
        "",
        "2.3.2.0",
        "",
        "2.3.4.0",
        "",
        "2.3.3.0",
        "",
        "2.3.5.0",
        "",
    },
    {
        "songs/thembones/thembones.mid",
        "songs/thembones/thembones.voc",
        "songs/thembones/thembones.mogg",
        "",
        "39.3.2.0",
        "",
        "39.3.4.0",
        "",
        "39.3.3.0",
        "",
        "39.3.5.0",
        "",
    },
    {
        "songs/warpigs/warpigs.mid",
        "songs/warpigs/warpigs.voc",
        "songs/warpigs/warpigs.mogg",
        "",
        "43.3.2.0",
        "",
        "43.3.4.0",
        "",
        "43.3.3.0",
        "",
        "43.3.5.0",
        "",
    },
    {
        "songs/monkeywrench/monkeywrench.mid",
        "songs/monkeywrench/monkeywrench.voc",
        "songs/monkeywrench/monkeywrench.mogg",
        "",
        "26.3.2.0",
        "",
        "26.3.4.0",
        "",
        "26.3.3.0",
        "",
        "26.3.5.0",
        "",
    },
    {
        "songs/hush/hush.mid",
        "songs/hush/hush.voc",
        "songs/hush/hush.mogg",
        "",
        "14.3.2.0",
        "",
        "14.3.4.0",
        "",
        "14.3.3.0",
        "",
        "14.3.5.0",
        "",
    },
    {
        "songs/girlfriend/girlfriend.mid",
        "songs/girlfriend/girlfriend.voc",
        "songs/girlfriend/girlfriend.mogg",
        "songs/girlfriend/girlfriend_coop.mogg",
        "11.3.2.0",
        "11.4.2.0",
        "11.3.4.0",
        "11.4.4.0",
        "11.3.3.0",
        "11.4.3.0",
        "11.3.5.0",
        "11.4.5.0",
    },
    {
        "songs/whowasinmyroom/whowasinmyroom.mid",
        "songs/whowasinmyroom/whowasinmyroom.voc",
        "songs/whowasinmyroom/whowasinmyroom.mogg",
        "songs/whowasinmyroom/whowasinmyroom_coop.mogg",
        "44.3.2.0",
        "44.4.2.0",
        "44.3.4.0",
        "44.4.4.0",
        "44.3.3.0",
        "44.4.3.0",
        "44.3.5.0",
        "44.4.5.0",
    },
    {
        "songs/cantyouhearme/cantyouhearme.mid",
        "songs/cantyouhearme/cantyouhearme.voc",
        "songs/cantyouhearme/cantyouhearme.mogg",
        "songs/cantyouhearme/cantyouhearme_coop.mogg",
        "3.3.2.0",
        "3.4.2.0",
        "3.3.4.0",
        "3.4.4.0",
        "3.3.3.0",
        "3.4.3.0",
        "3.3.5.0",
        "3.4.5.0",
    },
    {
        "songs/sweetchild/sweetchild.mid",
        "songs/sweetchild/sweetchild.voc",
        "songs/sweetchild/sweetchild.mogg",
        "",
        "37.3.2.0",
        "",
        "37.3.4.0",
        "",
        "37.3.3.0",
        "",
        "37.3.5.0",
        "",
    },
    {
        "songs/rockandroll/rockandroll.mid",
        "songs/rockandroll/rockandroll.voc",
        "songs/rockandroll/rockandroll.mogg",
        "",
        "29.3.2.0",
        "",
        "29.3.4.0",
        "",
        "29.3.3.0",
        "",
        "29.3.5.0",
        "",
    },
    {
        "songs/tattooedloveboys/tattooedloveboys.mid",
        "songs/tattooedloveboys/tattooedloveboys.voc",
        "songs/tattooedloveboys/tattooedloveboys.mogg",
        "",
        "38.3.2.0",
        "",
        "38.3.4.0",
        "",
        "38.3.3.0",
        "",
        "38.3.5.0",
        "",
    },
    {
        "songs/johnthefisherman/johnthefisherman.mid",
        "songs/johnthefisherman/johnthefisherman.voc",
        "songs/johnthefisherman/johnthefisherman.mogg",
        "",
        "17.4.2.0",
        "",
        "17.4.4.0",
        "",
        "17.4.3.0",
        "",
        "17.4.5.0",
        "",
    },
    {
        "songs/jessica/jessica.mid",
        "",
        "songs/jessica/jessica.mogg",
        "songs/jessica/jessica_coop.mogg",
        "16.3.2.0",
        "16.4.2.0",
        "16.3.4.0",
        "16.4.4.0",
        "16.3.3.0",
        "16.4.3.0",
        "16.3.5.0",
        "16.4.5.0",
    },
    {
        "songs/badreputation/badreputation.mid",
        "songs/badreputation/badreputation.voc",
        "songs/badreputation/badreputation.mogg",
        "",
        "0.3.2.0",
        "",
        "0.3.4.0",
        "",
        "0.3.3.0",
        "",
        "0.3.5.0",
        "",
    },
    {
        "songs/lastchild/lastchild.mid",
        "songs/lastchild/lastchild.voc",
        "songs/lastchild/lastchild.mogg",
        "songs/lastchild/lastchild_coop.mogg",
        "20.3.2.0",
        "20.4.2.0",
        "20.3.4.0",
        "20.4.4.0",
        "20.3.3.0",
        "20.4.3.0",
        "20.3.5.0",
        "20.4.5.0",
    },
    {
        "songs/crazyonyou/crazyonyou.mid",
        "songs/crazyonyou/crazyonyou.voc",
        "songs/crazyonyou/crazyonyou.mogg",
        "",
        "7.3.2.0",
        "",
        "7.3.4.0",
        "",
        "7.3.3.0",
        "",
        "7.3.5.0",
        "",
    },
    {
        "songs/trippinonahole/trippinonahole.mid",
        "songs/trippinonahole/trippinonahole.voc",
        "songs/trippinonahole/trippinonahole.mogg",
        "",
        "41.3.2.0",
        "",
        "41.3.4.0",
        "",
        "41.3.3.0",
        "",
        "41.3.5.0",
        "",
    },
    {
        "songs/dead/dead.mid",
        "songs/dead/dead.voc",
        "songs/dead/dead.mogg",
        "songs/dead/dead_coop.mogg",
        "8.4.2.0",
        "8.5.2.0",
        "8.4.4.0",
        "8.5.4.0",
        "8.4.3.0",
        "8.5.3.0",
        "8.4.5.0",
        "8.5.5.0",
    },
    {
        "songs/killinginthenameof/killinginthenameof.mid",
        "songs/killinginthenameof/killinginthenameof.voc",
        "songs/killinginthenameof/killinginthenameof.mogg",
        "",
        "18.3.2.0",
        "",
        "18.3.4.0",
        "",
        "18.3.3.0",
        "",
        "18.3.5.0",
        "",
    },
    {
        "songs/freya/freya.mid",
        "songs/freya/freya.voc",
        "songs/freya/freya.mogg",
        "",
        "10.3.2.0",
        "",
        "10.3.4.0",
        "",
        "10.3.3.0",
        "",
        "10.3.5.0",
        "",
    },
    {
        "songs/stop/stop.mid",
        "songs/stop/stop.voc",
        "songs/stop/stop.mogg",
        "",
        "34.4.2.0",
        "",
        "34.4.4.0",
        "",
        "34.4.3.0",
        "",
        "34.4.5.0",
        "",
    },
    {
        "songs/madhouse/madhouse.mid",
        "songs/madhouse/madhouse.voc",
        "songs/madhouse/madhouse.mogg",
        "",
        "22.3.2.0",
        "",
        "22.3.4.0",
        "",
        "22.3.3.0",
        "",
        "22.3.5.0",
        "",
    },
    {
        "songs/trooper/trooper.mid",
        "songs/trooper/trooper.voc",
        "songs/trooper/trooper.mogg",
        "",
        "42.3.2.0",
        "",
        "42.3.4.0",
        "",
        "42.3.3.0",
        "",
        "42.3.5.0",
        "",
    },
    {
        "songs/rockthistown/rockthistown.mid",
        "songs/rockthistown/rockthistown.voc",
        "songs/rockthistown/rockthistown.mogg",
        "",
        "30.3.2.0",
        "",
        "30.3.4.0",
        "",
        "30.3.3.0",
        "",
        "30.3.5.0",
        "",
    },
    {
        "songs/laidtorest/laidtorest.mid",
        "songs/laidtorest/laidtorest.voc",
        "songs/laidtorest/laidtorest.mogg",
        "songs/laidtorest/laidtorest_coop.mogg",
        "19.3.2.0",
        "19.4.2.0",
        "19.3.4.0",
        "19.4.4.0",
        "19.3.3.0",
        "19.4.3.0",
        "19.3.5.0",
        "19.4.5.0",
    },
    {
        "songs/psychobilly/psychobilly.mid",
        "songs/psychobilly/psychobilly.voc",
        "songs/psychobilly/psychobilly.mogg",
        "",
        "28.3.2.0",
        "",
        "28.3.4.0",
        "",
        "28.3.3.0",
        "",
        "28.3.5.0",
        "",
    },
    {
        "songs/yyz/yyz.mid",
        "",
        "songs/yyz/yyz.mogg",
        "",
        "47.3.2.0",
        "",
        "47.3.4.0",
        "",
        "47.3.3.0",
        "",
        "47.3.5.0",
        "",
    },
    {
        "songs/beastandtheharlot/beastandtheharlot.mid",
        "songs/beastandtheharlot/beastandtheharlot.voc",
        "songs/beastandtheharlot/beastandtheharlot.mogg",
        "",
        "1.3.2.0",
        "",
        "1.3.4.0",
        "",
        "1.3.3.0",
        "",
        "1.3.5.0",
        "",
    },
    {
        "songs/carrymehome/carrymehome.mid",
        "songs/carrymehome/carrymehome.voc",
        "songs/carrymehome/carrymehome.mogg",
        "",
        "4.3.2.0",
        "",
        "4.3.4.0",
        "",
        "4.3.3.0",
        "",
        "4.3.5.0",
        "",
    },
    {
        "songs/institutionalized/institutionalized.mid",
        "songs/institutionalized/institutionalized.voc",
        "songs/institutionalized/institutionalized.mogg",
        "songs/institutionalized/institutionalized_coop.mogg",
        "15.3.2.0",
        "15.4.2.0",
        "15.3.4.0",
        "15.4.4.0",
        "15.3.3.0",
        "15.4.3.0",
        "15.3.5.0",
        "15.4.5.0",
    },
    {
        "songs/misirlou/misirlou.mid",
        "",
        "songs/misirlou/misirlou.mogg",
        "",
        "24.3.2.0",
        "",
        "24.3.4.0",
        "",
        "24.3.3.0",
        "",
        "24.3.5.0",
        "",
    },
    {
        "songs/hangar18/hangar18.mid",
        "songs/hangar18/hangar18.voc",
        "songs/hangar18/hangar18.mogg",
        "",
        "12.3.2.0",
        "",
        "12.3.4.0",
        "",
        "12.3.3.0",
        "",
        "12.3.5.0",
        "",
    },
    {
        "songs/freebird/freebird.mid",
        "songs/freebird/freebird.voc",
        "songs/freebird/freebird.mogg",
        "songs/freebird/freebird_coop.mogg",
        "9.3.2.0",
        "9.4.2.0",
        "9.3.4.0",
        "9.4.4.0",
        "9.3.3.0",
        "9.4.3.0",
        "9.3.5.0",
        "9.4.5.0",
    },
    {
        "songs/rawdog/rawdog.mid",
        "songs/rawdog/rawdog.voc",
        "songs/rawdog/rawdog.mogg",
        "",
        "65.4.2.0",
        "",
        "65.4.4.0",
        "",
        "65.4.3.0",
        "",
        "65.4.5.0",
        "",
    },
    {
        "songs/arterialblack/arterialblack.mid",
        "songs/arterialblack/arterialblack.voc",
        "songs/arterialblack/arterialblack_sp.mogg",
        "songs/arterialblack/arterialblack_coop.mogg",
        "48.4.2.0",
        "48.5.2.0",
        "48.4.4.0",
        "48.5.4.0",
        "48.4.3.0",
        "48.5.3.0",
        "48.4.5.0",
        "48.5.5.0",
    },
    {
        "songs/collide/collide.mid",
        "songs/collide/collide.voc",
        "songs/collide/collide.mogg",
        "",
        "49.4.2.0",
        "",
        "49.4.4.0",
        "",
        "49.4.3.0",
        "",
        "49.4.5.0",
        "",
    },
    {
        "songs/drinkup/drinkup.mid",
        "songs/drinkup/drinkup.voc",
        "songs/drinkup/drinkup.mogg",
        "",
        "72.4.2.0",
        "",
        "72.4.4.0",
        "",
        "72.4.3.0",
        "",
        "72.4.5.0",
        "",
    },
    {
        "songs/elephantbones/elephantbones.mid",
        "songs/elephantbones/elephantbones.voc",
        "songs/elephantbones/elephantbones.mogg",
        "",
        "50.4.2.0",
        "",
        "50.4.4.0",
        "",
        "50.4.3.0",
        "",
        "50.4.5.0",
        "",
    },
    {
        "songs/fallofpangea/fallofpangea.mid",
        "songs/fallofpangea/fallofpangea.voc",
        "songs/fallofpangea/fallofpangea.mogg",
        "",
        "51.4.2.0",
        "",
        "51.4.4.0",
        "",
        "51.4.3.0",
        "",
        "51.4.5.0",
        "",
    },
    {
        "songs/ftk/ftk.mid",
        "songs/ftk/ftk.voc",
        "songs/ftk/ftk.mogg",
        "",
        "52.4.2.0",
        "",
        "52.4.4.0",
        "",
        "52.4.3.0",
        "",
        "52.4.5.0",
        "",
    },
    {
        "songs/gemini/gemini.mid",
        "",
        "songs/gemini/gemini.mogg",
        "",
        "53.4.2.0",
        "",
        "53.4.4.0",
        "",
        "53.4.3.0",
        "",
        "53.4.5.0",
        "",
    },
    {
        "songs/kicked/kicked.mid",
        "songs/kicked/kicked.voc",
        "songs/kicked/kicked.mogg",
        "",
        "55.4.2.0",
        "",
        "55.4.4.0",
        "",
        "55.4.3.0",
        "",
        "55.4.5.0",
        "",
    },
    {
        "songs/ladylightning/ladylightning.mid",
        "songs/ladylightning/ladylightning.voc",
        "songs/ladylightning/ladylightning.mogg",
        "",
        "56.4.2.0",
        "",
        "56.4.4.0",
        "",
        "56.4.3.0",
        "",
        "56.4.5.0",
        "",
    },
    {
        "songs/laughtrack/laughtrack.mid",
        "songs/laughtrack/laughtrack.voc",
        "songs/laughtrack/laughtrack.mogg",
        "songs/laughtrack/laughtrack_coop.mogg",
        "57.4.2.0",
        "57.5.2.0",
        "57.4.4.0",
        "57.5.4.0",
        "57.4.3.0",
        "57.5.3.0",
        "57.4.5.0",
        "57.5.5.0",
    },
    {
        "songs/lesstalkmorerokk/lesstalkmorerokk.mid",
        "songs/lesstalkmorerokk/lesstalkmorerokk.voc",
        "songs/lesstalkmorerokk/lesstalkmorerokk.mogg",
        "songs/lesstalkmorerokk/lesstalkmorerokk_coop.mogg",
        "58.4.2.0",
        "58.5.2.0",
        "58.4.4.0",
        "58.5.4.0",
        "58.4.3.0",
        "58.5.3.0",
        "58.4.5.0",
        "58.5.5.0",
    },
    {
        "songs/jordan/jordan.mid",
        "",
        "songs/jordan/jordan.mogg",
        "",
        "54.4.2.0",
        "",
        "54.4.4.0",
        "",
        "54.4.3.0",
        "",
        "54.4.5.0",
        "",
    },
    {
        "songs/mrfixit/mrfixit.mid",
        "songs/mrfixit/mrfixit.voc",
        "songs/mrfixit/mrfixit.mogg",
        "",
        "59.4.2.0",
        "",
        "59.4.4.0",
        "",
        "59.4.3.0",
        "",
        "59.4.5.0",
        "",
    },
    {
        "songs/newblack/newblack.mid",
        "songs/newblack/newblack.voc",
        "songs/newblack/newblack.mogg",
        "",
        "60.4.2.0",
        "",
        "60.4.4.0",
        "",
        "60.4.3.0",
        "",
        "60.4.5.0",
        "",
    },
    {
        "songs/onefortheroad/onefortheroad.mid",
        "songs/onefortheroad/onefortheroad.voc",
        "songs/onefortheroad/onefortheroad.mogg",
        "",
        "61.4.2.0",
        "",
        "61.4.4.0",
        "",
        "61.4.3.0",
        "",
        "61.4.5.0",
        "",
    },
    {
        "songs/parasite/parasite.mid",
        "songs/parasite/parasite.voc",
        "songs/parasite/parasite.mogg",
        "",
        "62.4.2.0",
        "",
        "62.4.4.0",
        "",
        "62.4.3.0",
        "",
        "62.4.5.0",
        "",
    },
    {
        "songs/radiumeyes/radiumeyes.mid",
        "songs/radiumeyes/radiumeyes.voc",
        "songs/radiumeyes/radiumeyes.mogg",
        "",
        "63.4.2.0",
        "",
        "63.4.4.0",
        "",
        "63.4.3.0",
        "",
        "63.4.5.0",
        "",
    },
    {
        "songs/redlottery/redlottery.mid",
        "songs/redlottery/redlottery.voc",
        "songs/redlottery/redlottery.mogg",
        "",
        "64.4.2.0",
        "",
        "64.4.4.0",
        "",
        "64.4.3.0",
        "",
        "64.4.5.0",
        "",
    },
    {
        "songs/six/six.mid",
        "songs/six/six.voc",
        "songs/six/six.mogg",
        "",
        "66.4.2.0",
        "",
        "66.4.4.0",
        "",
        "66.4.3.0",
        "",
        "66.4.5.0",
        "",
    },
    {
        "songs/soybomb/soybomb.mid",
        "",
        "songs/soybomb/soybomb.mogg",
        "songs/soybomb/soybomb_coop.mogg",
        "67.4.2.0",
        "67.5.2.0",
        "67.4.4.0",
        "67.5.4.0",
        "67.4.3.0",
        "67.5.3.0",
        "67.4.5.0",
        "67.5.5.0",
    },
    {
        "songs/thelightthatblinds/thelightthatblinds.mid",
        "songs/thelightthatblinds/thelightthatblinds.voc",
        "songs/thelightthatblinds/thelightthatblinds.mogg",
        "",
        "68.4.2.0",
        "",
        "68.4.4.0",
        "",
        "68.4.3.0",
        "",
        "68.4.5.0",
        "",
    },
    {
        "songs/thunderhorse/thunderhorse.mid",
        "songs/thunderhorse/thunderhorse.voc",
        "songs/thunderhorse/thunderhorse.mogg",
        "",
        "69.4.2.0",
        "",
        "69.4.4.0",
        "",
        "69.4.3.0",
        "",
        "69.4.5.0",
        "",
    },
    {
        "songs/trogdor/trogdor.mid",
        "songs/trogdor/trogdor.voc",
        "songs/trogdor/trogdor.mogg",
        "songs/trogdor/trogdor_coop.mogg",
        "70.4.2.0",
        "70.5.2.0",
        "70.4.4.0",
        "70.5.4.0",
        "70.4.3.0",
        "70.5.3.0",
        "70.4.5.0",
        "70.5.5.0",
    },
    {
        "songs/xstream/xstream.mid",
        "songs/xstream/xstream.voc",
        "songs/xstream/xstream.mogg",
        "",
        "71.4.2.0",
        "",
        "71.4.4.0",
        "",
        "71.4.3.0",
        "",
        "71.4.5.0",
        "",
    },
    {
        "songs/yeswecan/yeswecan.mid",
        "songs/yeswecan/yeswecan.voc",
        "songs/yeswecan/yeswecan.mogg",
        "",
        "73.4.2.0",
        "",
        "73.4.4.0",
        "",
        "73.4.3.0",
        "",
        "73.4.5.0",
        "",
    }
} ;

typedef struct DTBSETTINGS_STRUCT
{
    wxString id ;
    int dtb_type ;
    int orig_int ;
    int new_int ;
    float orig_float ;
    float new_float ;
    wxString orig_str ;
    wxString new_str ;
} dtbsettings ;





dtbsettings dtb_easy_guitars_settings[] =
{
    { _T("20.2.1"),      0, 0,   0,   0.00000f, 0.00000f, _T("tour_5_star"),         _T("tour_passed") }, //easy guitars
    { _T("21.3.1"),      0, 0,   0,   0.00000f, 0.00000f, _T("kDifficultyMedium"),         _T("kDifficultyEasy") }, //easy guitars
    { _T("22.2.1"),      0, 0,   0,   0.00000f, 0.00000f, _T("tour_5_star"),         _T("tour_passed") }, //easy guitars
    { _T("22.3.1"),      0, 0,   0,   0.00000f, 0.00000f, _T("kDifficultyMedium"),         _T("kDifficultyEasy") }, //easy guitars
    { _T("23.3.1"),      0, 0,   0,   0.00000f, 0.00000f, _T("kDifficultyHard"),         _T("kDifficultyEasy") }, //easy guitars
    { _T("24.2.1"),      0, 0,   0,   0.00000f, 0.00000f, _T("tour_5_star"),         _T("tour_passed") }, //easy guitars
    { _T("24.3.1"),      0, 0,   0,   0.00000f, 0.00000f, _T("kDifficultyHard"),         _T("kDifficultyEasy") }, //easy guitars
    { _T("25.3.1"),      0, 0,   0,   0.00000f, 0.00000f, _T("kDifficultyExpert"),         _T("kDifficultyEasy") }, //easy guitars
    { _T("26.3.1"),      0, 0,   0,   0.00000f, 0.00000f, _T("kDifficultyExpert"),         _T("kDifficultyEasy") }, //easy guitars
    { _T("26.2.1"),      0, 0,   0,   0.00000f, 0.00000f, _T("tour_5_star"),         _T("tour_passed") }, //easy guitars
    { _T("32.2.1.1"),    1, 5,   1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //keep meter full
    { _T("33.2.1.1"),    1, 10,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //keep meter full
    { _T("34.2.1.1"),    1, 20,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //keep meter full
    { _T("35.2.1.1"),    1, 30,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //keep meter full
    { _T("36.2.1.1"),    1, 40,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //keep meter full
    { _T("37.2.1.1"),    1, 48,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //keep meter full
    { _T(""),            0, 0,   0,   0.00000f, 0.00000f, _T(""),                    _T("") },
} ;

dtbsettings dtb_encore_settings[] =
{
    { _T("0.18.1"),      0, 0,   0,   0.00000f, 0.00000f, _T("FALSE"),         _T("TRUE") }, //encore venues in quick play
    { _T(""),            0, 0,   0,   0.00000f, 0.00000f, _T(""),                    _T("") },
} ;

dtbsettings dtb_autoplay_settings[] =
{
    { _T("0.9.1"),        0, 0,   0,   0.00000f, 0.00000f, _T("FALSE"),         _T("TRUE") }, //force autoplay on all the time
    { _T(""),            0, 0,   0,   0.00000f, 0.00000f, _T(""),                    _T("") },
} ;

dtbsettings dtb_starpower_settings[] =
{
    { _T("3.2.5.1"),      2, 0,   0,   0.50000f, 0.01000f, _T(""),         _T("") }, //star power ready
    { _T(""),            0, 0,   0,   0.00000f, 0.00000f, _T(""),                    _T("") },
} ;

dtbsettings dtb_meterfull_settings[] =
{
    { _T("3.2.1.1"),      2, 0,   0,   0.00000f, 0.95000f, _T(""),         _T("") }, //meter full
    { _T("4.2.1.1"),      2, 0,   0,   0.00000f, 0.95000f, _T(""),         _T("") }, //meter full
    { _T(""),            0, 0,   0,   0.00000f, 0.00000f, _T(""),                    _T("") },
} ;

dtbsettings dtb_freyafix_settings[] =
{
    { _T("10.5.1"),      1, 256500,  30000,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("10.5.2"),      1, 277400,  50000,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T(""),            0, 0,   0,   0.00000f, 0.00000f, _T(""),                    _T("") },
};

dtbsettings dtb_unlocksave_settings[] =
{
    { _T("25.1.4.2"),      1, 1,  0,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("25.1.5.2"),      1, 1,  0,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T(""),            0, 0,   0,   0.00000f, 0.00000f, _T(""),                    _T("") },
};

dtbsettings dtb_freestore_settings[] =
{
    { _T("0.1.1.1"),    1, 700,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("0.2.1.1"),    1, 850,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("0.3.1.1"),    1, 700,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("0.4.1.1"),    1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("0.5.1.1"),    1, 700,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("0.6.1.1"),    1, 850,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("0.7.1.1"),    1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("0.8.1.1"),    1, 850,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("0.9.1.1"),    1, 700,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("0.10.1.1"),   1, 950,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("0.11.1.1"),   1, 700,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("0.12.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("0.13.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("0.14.1.1"),   1, 700,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("0.15.1.1"),   1, 700,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("0.16.1.1"),   1, 850,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("0.17.1.1"),   1, 500,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("0.18.1.1"),   1, 1000, 1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("0.19.1.1"),   1, 1000, 1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("0.20.1.1"),   1, 2000, 1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("0.21.1.1"),   1, 2000, 1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("0.22.1.1"),   1, 5000, 1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("0.23.1.1"),   1, 5600, 1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("0.24.1.1"),   1, 8000, 1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.1.1.1"),    1, 250,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.2.1.1"),    1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.3.1.1"),    1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.4.1.1"),    1, 250,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.5.1.1"),    1, 450,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.6.1.1"),    1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.7.1.1"),    1, 250,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.8.1.1"),    1, 450,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.9.1.1"),    1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.10.1.1"),   1, 250,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.11.1.1"),   1, 450,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.12.1.1"),   1, 450,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.13.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.14.1.1"),   1, 250,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.15.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.16.1.1"),   1, 250,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.17.1.1"),   1, 450,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.18.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.19.1.1"),   1, 250,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.20.1.1"),   1, 450,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.21.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.22.1.1"),   1, 450,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.23.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.24.1.1"),   1, 300,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.25.1.1"),   1, 450,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.26.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("1.27.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.1.1.1"),    1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.2.1.1"),    1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.3.1.1"),    1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.4.1.1"),    1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.5.1.1"),    1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.6.1.1"),    1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.7.1.1"),    1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.8.1.1"),    1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.9.1.1"),    1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.10.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.11.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.12.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.13.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.14.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.15.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.16.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.17.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.18.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.19.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.20.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.21.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.22.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.23.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.24.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.25.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("2.26.1.1"),   1, 550,  1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("3.1.1.1"),    1, 5000, 1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("3.2.1.1"),    1, 5000, 1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("3.3.1.1"),    1, 7000, 1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("4.1.1.1"),    1, 2000, 1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("4.2.1.1"),    1, 2000, 1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("4.3.1.1"),    1, 2000, 1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("4.4.1.1"),    1, 2000, 1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("4.5.1.1"),    1, 2000, 1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("4.6.1.1"),    1, 2000, 1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("4.7.1.1"),    1, 2000, 1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("4.8.1.1"),    1, 2000, 1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("5.1.7.1"),    1, 1500, 1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T("5.2.7.1"),    1, 1500, 1,   0.00000f, 0.00000f, _T(""),                    _T("") }, //free store
    { _T(""),            0, 0,   0,   0.00000f, 0.00000f, _T(""),                    _T("") },
} ;


#define OPENPARSEDTB( _fname, _dtbname ) \
    _fname = wxFileName::CreateTempFileName( _T("gh2makertmp") ) ;\
\
    if ( ! g_frame->m_arkhdr.GetFile( _fname.char_str(), _dtbname, true ) )\
    {\
        for ( int i = 0 ; i < 9 ; i++ ) { if ( dtb_filenames[i]->Length() ) ::wxRemoveFile( *dtb_filenames[i] ) ; }\
        Error( wxT( "Could not extract " ) wxT( _dtbname ) wxT(" from ARK") ) ;\
        return 0;\
    }\
\
    if ( dtb_read( _fname.char_str() ) )\
    {\
        for ( int i = 0 ; i < 9 ; i++ ) { if ( dtb_filenames[i]->Length() ) ::wxRemoveFile( *dtb_filenames[i] ) ; }\
        Error( wxT( "Could not parse ") wxT( _dtbname ) wxT(" from ARK") ) ;\
        return 0;\
    }

#define SETNEWDTB( _settings, _dtbname )\
    curr_setting = 0 ;\
\
    while( _settings[curr_setting].id.Length() )\
    {\
        switch( _settings[curr_setting].dtb_type )\
        {\
            case 0 :\
            {\
                if ( dtb_set_string( _settings[curr_setting].id.char_str(), _settings[curr_setting].new_str.char_str() ) )\
                {\
                    for ( int i = 0 ; i < 9 ; i++ ) { if ( dtb_filenames[i]->Length() ) ::wxRemoveFile( *dtb_filenames[i] ) ; }\
                    Error( wxT( "Could not set DTB data into ") wxT( _dtbname)  ) ;\
                    return 0;\
                }\
                break ;\
            }\
            case 1 :\
            {\
                if ( dtb_set_int( _settings[curr_setting].id.char_str(), _settings[curr_setting].new_int ) )\
                {\
                    for ( int i = 0 ; i < 9 ; i++ ) { if ( dtb_filenames[i]->Length() ) ::wxRemoveFile( *dtb_filenames[i] ) ; }\
                    Error( wxT( "Could not set DTB data into ") wxT( _dtbname)  ) ;\
                    return 0;\
                }\
                break ;\
            }\
            case 2 :\
            {\
                if ( dtb_set_float( _settings[curr_setting].id.char_str(), _settings[curr_setting].new_float ) )\
                {\
                    for ( int i = 0 ; i < 9 ; i++ ) { if ( dtb_filenames[i]->Length() ) ::wxRemoveFile( *dtb_filenames[i] ) ; }\
                    Error( wxT( "Could not set DTB data into ") wxT( _dtbname)  ) ;\
                    return 0;\
                }\
                break ;\
            }\
        }\
\
        curr_setting++ ;\
    }


#define SETOLDDTB( _settings, _dtbname )\
    curr_setting = 0 ;\
\
    while( _settings[curr_setting].id.Length() )\
    {\
        switch( _settings[curr_setting].dtb_type )\
        {\
            case 0 :\
            {\
                if ( dtb_set_string( _settings[curr_setting].id.char_str(), _settings[curr_setting].orig_str.char_str() ) )\
                {\
                    for ( int i = 0 ; i < 9 ; i++ ) { if ( dtb_filenames[i]->Length() ) ::wxRemoveFile( *dtb_filenames[i] ) ; }\
                    Error( wxT( "Could not set DTB data into ") wxT( _dtbname)  ) ;\
                    return 0;\
                }\
                break ;\
            }\
            case 1 :\
            {\
                if ( dtb_set_int( _settings[curr_setting].id.char_str(), _settings[curr_setting].orig_int ) )\
                {\
                    for ( int i = 0 ; i < 9 ; i++ ) { if ( dtb_filenames[i]->Length() ) ::wxRemoveFile( *dtb_filenames[i] ) ; }\
                    Error( wxT( "Could not set DTB data into ") wxT( _dtbname)  ) ;\
                    return 0;\
                }\
                break ;\
            }\
            case 2 :\
            {\
                if ( dtb_set_float( _settings[curr_setting].id.char_str(), _settings[curr_setting].orig_float ) )\
                {\
                    for ( int i = 0 ; i < 9 ; i++ ) { if ( dtb_filenames[i]->Length() ) ::wxRemoveFile( *dtb_filenames[i] ) ; }\
                    Error( wxT( "Could not set DTB data into ") wxT( _dtbname)  ) ;\
                    return 0;\
                }\
                break ;\
            }\
        }\
\
        curr_setting++ ;\
    }\



#define BOMB_OUT( _errmsg )\
    {\
        for ( int i = 0 ; i < 9 ; i++ ) { if ( dtb_filenames[i]->Length() ) ::wxRemoveFile( *dtb_filenames[i] ) ; }\
        Error( _errmsg ) ;\
        return 0;\
    }\



#define SET_DTB_GRID( _rownum, _colnum, _idstr )\
    if ( dtb_set_string( _idstr, g_frame->Grid1->GetCellValue( _rownum, _colnum ).char_str() ) )\
    {\
        BOMB_OUT( wxT( "Could not set DTB data into config/gen/songs.dtb" ) )\
    }\



wxThread::ExitCode FinalizerThread::Entry()
{

    g_semProgress.Wait() ;

    bool missing_files = false ;
    bool midi_problem = false;
    int  num_convert_midi = 0 ;
    int  num_convert_ogg = 0 ;
    int  curr_convert_midi = 0 ;
    int  curr_convert_ogg = 0 ;
    int  curr_dtb = 0 ;
    int  curr_setting = 0 ;
    bool asked_size = false ;

    wxString filename ;
    wxString filename_dtb_guitars(_T(""));
    wxString filename_dtb_modes(_T(""));
    wxString filename_dtb_scoring(_T(""));
    wxString filename_dtb_store(_T(""));
    wxString filename_dtb_track(_T(""));
    wxString filename_dtb_songs(_T(""));
    wxString filename_dtb_cheats(_T(""));
    wxString filename_dtb_beatmatcher(_T(""));
    wxString filename_dtb_locale(_T(""));
    wxString *dtb_filenames[9]  ;
    wxString dummy_midi_file = _T("");
    wxString dummy_ogg_file = _T("");

    dtb_filenames[0] = &filename_dtb_guitars ;
    dtb_filenames[1] = &filename_dtb_modes;
    dtb_filenames[2] = &filename_dtb_scoring;
    dtb_filenames[3] = &filename_dtb_store;
    dtb_filenames[4] = &filename_dtb_track;
    dtb_filenames[5] = &filename_dtb_songs;
    dtb_filenames[6] = &filename_dtb_cheats;
    dtb_filenames[7] = &filename_dtb_beatmatcher;
    dtb_filenames[8] = &filename_dtb_locale;

    m_current_step = 0 ;
    m_num_injections = 0 ;


    if( !g_frame->m_arkhdr.Open( g_frame->m_arkdir.char_str() ) )
    {
        Error ( wxT( "ARK Directory is invalid" ) ) ;
        return 0;
    }

    for ( int i = 0 ; i < 74 ; i++ )
    {
        if ( ( g_frame->m_valid_midi[i]==1 ) && ( g_frame->m_construct_midi[i] ) )
            num_convert_midi++ ;
        if ( ( g_frame->m_valid_ogg[i]==1 ) && ( g_frame->m_construct_ogg[i] ) )
            num_convert_ogg++ ;
        if ( g_frame->m_valid_midi[i] > 0 )
            m_num_injections++ ;
        if ( g_frame->m_valid_ogg[i] > 0 )
        {
                m_num_injections++ ;

                if ( strlen( song_filenames[i].coop ) )
                    m_num_injections++ ;
        }
    }

    // +1 = scanning existence of all files
    // +9 = injecting 9 DTB files
    // +9 = reformatting 9 DTB files
    m_total_steps = num_convert_midi + num_convert_ogg + m_num_injections + 1 + 9 + 9 ;

    OverallProgress( m_current_step++, m_total_steps, wxString( _T("Checking existence of files") ) ) ;

    if ( m_notify_dlg->m_done )
    {
        BOMB_OUT( wxT("Cancelled" ) )
    }

    for ( int i = 0 ; i < 74 ; i++ )
    {
        CurrentProgress( i, 74, wxString::Format( _T("Checking Line %d"), i ) )  ;
        if ( g_frame->m_valid_midi[i] == 1  )
        {
            if ( ! wxFileExists( g_frame->m_midifile[i] ) )
            {
                missing_files = true ;
                g_crit.Enter() ;
                if ( m_notify_dlg->m_done )
                {
                    g_crit.Leave() ;
                    BOMB_OUT( wxT("Cancelled" ) ) ;
                }
                wxMutexGuiEnter();
                g_frame->Grid1->SetCellBackgroundColour( i, 2, wxColor( 0x80, 0xA0, 0xF0 ) ) ;
                wxMutexGuiLeave() ;
                g_crit.Leave() ;
            }
        }

        if ( g_frame->m_valid_ogg[i] == 1 )
        {
            if ( ( !g_frame->m_construct_ogg[i] ) && ( ! wxFileExists( g_frame->m_oggfile[i] ) ) )
            {
                missing_files = true ;
                g_crit.Enter() ;
                if ( m_notify_dlg->m_done )
                {
                    g_crit.Leave() ;
                    BOMB_OUT( wxT("Cancelled" ) ) ;
                }
                wxMutexGuiEnter();
                g_frame->Grid1->SetCellBackgroundColour( i, 3, wxColor( 0x80, 0xA0, 0xF0 ) ) ;
                wxMutexGuiLeave() ;
                g_crit.Leave() ;
            }

            if ( g_frame->m_construct_ogg[i] )
            {
                if ( ( g_frame->m_construct_ogg_source_files[i][0].Length() == 0 ) &&
                     ( g_frame->m_construct_ogg_source_files[i][1].Length() == 0 ) &&
                     ( g_frame->m_construct_ogg_source_files[i][2].Length() == 0 ) )
                {
                    missing_files = true ;
                    g_crit.Enter() ;
                    if ( m_notify_dlg->m_done )
                    {
                        g_crit.Leave() ;
                        BOMB_OUT( wxT("Cancelled" ) ) ;
                    }
                    wxMutexGuiEnter();
                    g_frame->Grid1->SetCellBackgroundColour( i, 3, wxColor( 0x80, 0xA0, 0xF0 ) ) ;
                    wxMutexGuiLeave() ;
                    g_crit.Leave() ;
                }

                for ( int j = 0 ; j < 3 ; j++ )
                {
                    if ( ( g_frame->m_construct_ogg_source_files[i][j].Length() > 0 ) && ( ! wxFileExists( g_frame->m_construct_ogg_source_files[i][j] ) ) )
                    {
                        missing_files = true ;
                        g_crit.Enter() ;
                        if ( m_notify_dlg->m_done )
                        {
                            g_crit.Leave() ;
                            BOMB_OUT( wxT("Cancelled" ) ) ;
                        }
                        wxMutexGuiEnter();
                        g_frame->Grid1->SetCellBackgroundColour( i, 3, wxColor( 0x80, 0xA0, 0xF0 ) ) ;
                        wxMutexGuiLeave() ;
                        g_crit.Leave() ;
                    }
                }
            }
        }
    }

    if ( m_notify_dlg->m_done )
    {
        BOMB_OUT( wxT("Cancelled" ) ) ;
    }
    if ( missing_files )
    {
        BOMB_OUT( wxT( "Some files are missing that existed in the past.  Those boxes have been colored blue.  Please fix them before continuing.") )
    }


    curr_dtb = 0 ;
    OverallProgress( m_current_step++, m_total_steps, wxString::Format( _T("Reconfiguring %d of 9 DTB files"), ++curr_dtb  ) )  ;
    CurrentProgress( 1, 1, wxString::Format( _T("%s"), _T("config/gen/guitars.dtb") ) )  ;

    OPENPARSEDTB( filename_dtb_guitars, "config/gen/guitars.dtb" )

    if ( g_frame->m_dtb_easy_unlock_guitars )
    {
        SETNEWDTB( dtb_easy_guitars_settings, "config/gen/guitars.dtb" )
    }
    else
    {
        SETOLDDTB( dtb_easy_guitars_settings, "config/gen/guitars.dtb" )
    }
    if ( dtb_write( filename_dtb_guitars.char_str() ) )
    {
        BOMB_OUT( wxT( "Could not write new ") wxT( "config/gen/guitars.dtb")  )
    }


    if ( m_notify_dlg->m_done )
    {
        BOMB_OUT( wxT("Cancelled" ) ) ;
    }
    OverallProgress( m_current_step++, m_total_steps, wxString::Format( _T("Reconfiguring %d of 9 DTB files"), ++curr_dtb  ) )  ;
    CurrentProgress( 1, 1, wxString::Format( _T("%s"), _T("config/gen/modes.dtb") ) )  ;


    OPENPARSEDTB( filename_dtb_modes, "config/gen/modes.dtb" )

    if ( g_frame->m_dtb_encore_venues )
    {
        SETNEWDTB( dtb_encore_settings, "config/gen/modes.dtb" )
    }
    else
    {
        SETOLDDTB( dtb_encore_settings, "config/gen/modes.dtb" )
    }

    if ( dtb_write( filename_dtb_modes.char_str() ) )
    {
        BOMB_OUT( wxT( "Could not write new ") wxT( "config/gen/modes.dtb")  )
    }


    if ( m_notify_dlg->m_done )
    {
        BOMB_OUT( wxT("Cancelled" ) ) ;
    }
    OverallProgress( m_current_step++, m_total_steps, wxString::Format( _T("Reconfiguring %d of 9 DTB files"), ++curr_dtb  ) )  ;
    CurrentProgress( 1, 1, wxString::Format( _T("%s"), _T("config/gen/scoring.dtb") ) )  ;


    OPENPARSEDTB( filename_dtb_scoring, "config/gen/scoring.dtb" )

    if ( g_frame->m_dtb_always_star_power )
    {
        SETNEWDTB( dtb_starpower_settings, "config/gen/scoring.dtb" )
    }
    else
    {
        SETOLDDTB( dtb_starpower_settings, "config/gen/scoring.dtb" )
    }

    if ( g_frame->m_dtb_always_star_power )
    {
        SETNEWDTB( dtb_meterfull_settings, "config/gen/scoring.dtb" )
    }
    else
    {
        SETOLDDTB( dtb_meterfull_settings, "config/gen/scoring.dtb" )
    }

    if ( dtb_set_int( "0.1.1", g_frame->m_dtb_note_points ) )
    {
        BOMB_OUT( wxT( "Could not set DTB data into config/gen/scoring.dtb" ) )
    }
    if ( dtb_set_int( "0.2.1", g_frame->m_dtb_sustain_points ) )
    {
        BOMB_OUT( wxT( "Could not set DTB data into config/gen/scoring.dtb" ) )
    }
    if ( dtb_write( filename_dtb_scoring.char_str() ) )
    {
        BOMB_OUT( wxT( "Could not write new ") wxT( "config/gen/scoring.dtb")  )
    }


    if ( m_notify_dlg->m_done )
    {
        BOMB_OUT( wxT("Cancelled" ) ) ;
    }
    OverallProgress( m_current_step++, m_total_steps, wxString::Format( _T("Reconfiguring %d of 9 DTB files"), ++curr_dtb  ) )  ;
    CurrentProgress( 1, 1, wxString::Format( _T("%s"), _T("config/gen/store.dtb" )) )  ;


    OPENPARSEDTB( filename_dtb_store, "config/gen/store.dtb" )

    if ( g_frame->m_dtb_free_store )
    {
        SETNEWDTB( dtb_freestore_settings, "config/gen/store.dtb" )
    }
    else
    {
        SETOLDDTB( dtb_freestore_settings, "config/gen/store.dtb" )
    }

    if ( dtb_write( filename_dtb_store.char_str() ) )
    {
        BOMB_OUT( wxT( "Could not write new ") wxT( "config/gen/store.dtb")  )
    }


    if ( m_notify_dlg->m_done )
    {
        BOMB_OUT( wxT("Cancelled" ) ) ;
    }
    OverallProgress( m_current_step++, m_total_steps, wxString::Format( _T("Reconfiguring %d of 9 DTB files"), ++curr_dtb  ) )  ;
    CurrentProgress( 1, 1, wxString::Format( _T("%s"), _T("config/gen/track_graphics.dtb" )) )  ;


    OPENPARSEDTB( filename_dtb_track, "config/gen/track_graphics.dtb" )

    if ( dtb_set_float( "9.3.1", ((float)(g_frame->m_dtb_track_speed_hard))/1000000.0f ) )
    {
        BOMB_OUT( wxT( "Could not set DTB data into config/gen/track_graphics.dtb" ) )
    }
    if ( dtb_set_float( "9.4.1", ((float)(g_frame->m_dtb_track_speed_expert))/1000000.0f ) )
    {
        BOMB_OUT( wxT( "Could not set DTB data into config/gen/track_graphics.dtb" ) )
    }
    if ( dtb_write( filename_dtb_track.char_str() ) )
    {
        BOMB_OUT( wxT( "Could not write new ") wxT( "config/gen/track_graphics.dtb")  )
    }


    if ( m_notify_dlg->m_done )
    {
        BOMB_OUT( wxT("Cancelled" ) ) ;
    }
    OverallProgress( m_current_step++, m_total_steps, wxString::Format( _T("Reconfiguring %d of 9 DTB files"), ++curr_dtb  ) )  ;
    CurrentProgress( 1, 1, wxString::Format( _T("%s"), _T("config/gen/songs.dtb") ) )  ;


    OPENPARSEDTB( filename_dtb_songs, "config/gen/songs.dtb" )

    if ( dtb_set_int( "10.5.1", 30000 ) )
    {
        BOMB_OUT( wxT( "Could not set DTB data into config/gen/songs.dtb" ) )
    }
    if ( dtb_set_int( "10.5.2", 50000 ) )
    {
        BOMB_OUT( wxT( "Could not set DTB data into config/gen/songs.dtb" ) )
    }

    if ( dtb_set_string( "36.1.1", g_frame->Grid1->GetCellValue( 0, 0 ).char_str() ) )
    {
        BOMB_OUT( wxT( "Could not set DTB data into config/gen/songs.dtb" ) )
    }
    if ( dtb_set_string( "53.4.2.1.1.0", "bass" ) )
    {
        BOMB_OUT( wxT( "Could not set DTB data into config/gen/songs.dtb" ) )
    }

    for ( int i = 0 ; i < 74 ; i++ )
    {
        if ( ( g_frame->m_valid_ogg[i] == 1 )  )
        {
            dtb_fixup_track( song_filenames[i].trackid ) ;
            if ( song_filenames[i].trackid_coop[0] )
                dtb_fixup_track( song_filenames[i].trackid_coop ) ;

            dtb_fixup_vols( song_filenames[i].vols ) ;
            if ( song_filenames[i].vols_coop[0] )
                dtb_fixup_vols( song_filenames[i].vols_coop) ;

            dtb_fixup_pans( song_filenames[i].pans ) ;
            if ( song_filenames[i].pans_coop[0] )
                dtb_fixup_pans( song_filenames[i].pans_coop) ;

            dtb_fixup_cores( song_filenames[i].cores ) ;
            if ( song_filenames[i].cores_coop[0] )
                dtb_fixup_cores( song_filenames[i].cores_coop) ;

        }
    }

//    wxCriticalSectionLocker dtblocker( g_crit_grid ) ;
    g_crit_grid.Enter() ;

    if ( m_notify_dlg->m_done )
    {
        BOMB_OUT( wxT("Cancelled" ) ) ;
    }
    wxMutexGuiEnter();

    SET_DTB_GRID( 0, 0, "36.1.1" )
    SET_DTB_GRID( 0, 1, "36.2.1" )
    SET_DTB_GRID( 1, 0, "27.1.1" )
    SET_DTB_GRID( 1, 1, "27.2.1" )
    SET_DTB_GRID( 2, 0, "13.1.1" )
    SET_DTB_GRID( 2, 1, "13.2.1" )
    SET_DTB_GRID( 3, 0, "31.1.1" )
    SET_DTB_GRID( 3, 1, "31.2.1" )
    SET_DTB_GRID( 4, 0, "35.1.1" )
    SET_DTB_GRID( 4, 1, "35.2.1" )
    SET_DTB_GRID( 5, 0, "33.1.1" )
    SET_DTB_GRID( 5, 1, "33.2.1" )
    SET_DTB_GRID( 6, 0, "25.1.1" )
    SET_DTB_GRID( 6, 1, "25.2.1" )
    SET_DTB_GRID( 7, 0, "21.1.1" )
    SET_DTB_GRID( 7, 1, "21.2.1" )
    SET_DTB_GRID( 8, 0, "6.1.1" )
    SET_DTB_GRID( 8, 1, "6.2.1" )
    SET_DTB_GRID( 9, 0, "45.1.1" )
    SET_DTB_GRID( 9, 1, "45.2.1" )
    SET_DTB_GRID( 10, 0, "46.1.1" )
    SET_DTB_GRID( 10, 1, "46.2.1" )
    SET_DTB_GRID( 11, 0, "40.1.1" )
    SET_DTB_GRID( 11, 1, "40.2.1" )
    SET_DTB_GRID( 12, 0, "5.1.1" )
    SET_DTB_GRID( 12, 1, "5.2.1" )
    SET_DTB_GRID( 13, 0, "32.1.1" )
    SET_DTB_GRID( 13, 1, "32.2.1" )
    SET_DTB_GRID( 14, 0, "23.1.1" )
    SET_DTB_GRID( 14, 1, "23.2.1" )
    SET_DTB_GRID( 15, 0, "2.1.1" )
    SET_DTB_GRID( 15, 1, "2.2.1" )
    SET_DTB_GRID( 16, 0, "39.1.1" )
    SET_DTB_GRID( 16, 1, "39.2.1" )
    SET_DTB_GRID( 17, 0, "43.1.1" )
    SET_DTB_GRID( 17, 1, "43.2.1" )
    SET_DTB_GRID( 18, 0, "26.1.1" )
    SET_DTB_GRID( 18, 1, "26.2.1" )
    SET_DTB_GRID( 19, 0, "14.1.1" )
    SET_DTB_GRID( 19, 1, "14.2.1" )
    SET_DTB_GRID( 20, 0, "11.1.1" )
    SET_DTB_GRID( 20, 1, "11.2.1" )
    SET_DTB_GRID( 21, 0, "44.1.1" )
    SET_DTB_GRID( 21, 1, "44.2.1" )
    SET_DTB_GRID( 22, 0, "3.1.1" )
    SET_DTB_GRID( 22, 1, "3.2.1" )
    SET_DTB_GRID( 23, 0, "37.1.1" )
    SET_DTB_GRID( 23, 1, "37.2.1" )
    SET_DTB_GRID( 24, 0, "29.1.1" )
    SET_DTB_GRID( 24, 1, "29.2.1" )
    SET_DTB_GRID( 25, 0, "38.1.1" )
    SET_DTB_GRID( 25, 1, "38.2.1" )
    SET_DTB_GRID( 26, 0, "17.1.1" )
    SET_DTB_GRID( 26, 1, "17.2.1" )
    SET_DTB_GRID( 27, 0, "16.1.1" )
    SET_DTB_GRID( 27, 1, "16.2.1" )
    SET_DTB_GRID( 28, 0, "0.1.1" )
    SET_DTB_GRID( 28, 1, "0.2.1" )
    SET_DTB_GRID( 29, 0, "20.1.1" )
    SET_DTB_GRID( 29, 1, "20.2.1" )
    SET_DTB_GRID( 30, 0, "7.1.1" )
    SET_DTB_GRID( 30, 1, "7.2.1" )
    SET_DTB_GRID( 31, 0, "41.1.1" )
    SET_DTB_GRID( 31, 1, "41.2.1" )
    SET_DTB_GRID( 32, 0, "8.1.1" )
    SET_DTB_GRID( 32, 1, "8.2.1" )
    SET_DTB_GRID( 33, 0, "18.1.1" )
    SET_DTB_GRID( 33, 1, "18.2.1" )
    SET_DTB_GRID( 34, 0, "10.1.1" )
    SET_DTB_GRID( 34, 1, "10.2.1" )
    SET_DTB_GRID( 35, 0, "34.1.1" )
    SET_DTB_GRID( 35, 1, "34.2.1" )
    SET_DTB_GRID( 36, 0, "22.1.1" )
    SET_DTB_GRID( 36, 1, "22.2.1" )
    SET_DTB_GRID( 37, 0, "42.1.1" )
    SET_DTB_GRID( 37, 1, "42.2.1" )
    SET_DTB_GRID( 38, 0, "30.1.1" )
    SET_DTB_GRID( 38, 1, "30.2.1" )
    SET_DTB_GRID( 39, 0, "19.1.1" )
    SET_DTB_GRID( 39, 1, "19.2.1" )
    SET_DTB_GRID( 40, 0, "28.1.1" )
    SET_DTB_GRID( 40, 1, "28.2.1" )
    SET_DTB_GRID( 41, 0, "47.1.1" )
    SET_DTB_GRID( 41, 1, "47.2.1" )
    SET_DTB_GRID( 42, 0, "1.1.1" )
    SET_DTB_GRID( 42, 1, "1.2.1" )
    SET_DTB_GRID( 43, 0, "4.1.1" )
    SET_DTB_GRID( 43, 1, "4.2.1" )
    SET_DTB_GRID( 44, 0, "15.1.1" )
    SET_DTB_GRID( 44, 1, "15.2.1" )
    SET_DTB_GRID( 45, 0, "24.1.1" )
    SET_DTB_GRID( 45, 1, "24.2.1" )
    SET_DTB_GRID( 46, 0, "12.1.1" )
    SET_DTB_GRID( 46, 1, "12.2.1" )
    SET_DTB_GRID( 47, 0, "9.1.1" )
    SET_DTB_GRID( 47, 1, "9.2.1" )
    SET_DTB_GRID( 48, 0, "65.1.1" )
    SET_DTB_GRID( 48, 1, "65.2.1" )
    SET_DTB_GRID( 49, 0, "48.1.1" )
    SET_DTB_GRID( 49, 1, "48.2.1" )
    SET_DTB_GRID( 50, 0, "49.1.1" )
    SET_DTB_GRID( 50, 1, "49.2.1" )
    SET_DTB_GRID( 51, 0, "72.1.1" )
    SET_DTB_GRID( 51, 1, "72.2.1" )
    SET_DTB_GRID( 52, 0, "50.1.1" )
    SET_DTB_GRID( 52, 1, "50.2.1" )
    SET_DTB_GRID( 53, 0, "51.1.1" )
    SET_DTB_GRID( 53, 1, "51.2.1" )
    SET_DTB_GRID( 54, 0, "52.1.1" )
    SET_DTB_GRID( 54, 1, "52.2.1" )
    SET_DTB_GRID( 55, 0, "53.1.1" )
    SET_DTB_GRID( 55, 1, "53.2.1" )
    SET_DTB_GRID( 56, 0, "55.1.1" )
    SET_DTB_GRID( 56, 1, "55.2.1" )
    SET_DTB_GRID( 57, 0, "56.1.1" )
    SET_DTB_GRID( 57, 1, "56.2.1" )
    SET_DTB_GRID( 58, 0, "57.1.1" )
    SET_DTB_GRID( 58, 1, "57.2.1" )
    SET_DTB_GRID( 59, 0, "58.1.1" )
    SET_DTB_GRID( 59, 1, "58.2.1" )
    SET_DTB_GRID( 60, 0, "54.1.1" )
    SET_DTB_GRID( 60, 1, "54.2.1" )
    SET_DTB_GRID( 61, 0, "59.1.1" )
    SET_DTB_GRID( 61, 1, "59.2.1" )
    SET_DTB_GRID( 62, 0, "60.1.1" )
    SET_DTB_GRID( 62, 1, "60.2.1" )
    SET_DTB_GRID( 63, 0, "61.1.1" )
    SET_DTB_GRID( 63, 1, "61.2.1" )
    SET_DTB_GRID( 64, 0, "62.1.1" )
    SET_DTB_GRID( 64, 1, "62.2.1" )
    SET_DTB_GRID( 65, 0, "63.1.1" )
    SET_DTB_GRID( 65, 1, "63.2.1" )
    SET_DTB_GRID( 66, 0, "64.1.1" )
    SET_DTB_GRID( 66, 1, "64.2.1" )
    SET_DTB_GRID( 67, 0, "66.1.1" )
    SET_DTB_GRID( 67, 1, "66.2.1" )
    SET_DTB_GRID( 68, 0, "67.1.1" )
    SET_DTB_GRID( 68, 1, "67.2.1" )
    SET_DTB_GRID( 69, 0, "68.1.1" )
    SET_DTB_GRID( 69, 1, "68.2.1" )
    SET_DTB_GRID( 70, 0, "69.1.1" )
    SET_DTB_GRID( 70, 1, "69.2.1" )
    SET_DTB_GRID( 71, 0, "70.1.1" )
    SET_DTB_GRID( 71, 1, "70.2.1" )
    SET_DTB_GRID( 72, 0, "71.1.1" )
    SET_DTB_GRID( 72, 1, "71.2.1" )
    SET_DTB_GRID( 73, 0, "73.1.1" )
    SET_DTB_GRID( 73, 1, "73.2.1" )

    wxMutexGuiLeave();
    g_crit_grid.Leave() ;


    if ( dtb_write( filename_dtb_songs.char_str() ) )
    {
        BOMB_OUT( wxT( "Could not write new ") wxT( "config/gen/songs.dtb")  )
    }


    if ( m_notify_dlg->m_done )
    {
        BOMB_OUT( wxT("Cancelled" ) ) ;
    }
    OverallProgress( m_current_step++, m_total_steps, wxString::Format( _T("Reconfiguring %d of 9 DTB files"), ++curr_dtb  ) )  ;
    CurrentProgress( 1, 1, wxString::Format( _T("%s"), _T("config/gen/long_cheats.dtb") ) )  ;


    OPENPARSEDTB( filename_dtb_cheats, "config/gen/long_cheats.dtb" )

    if ( g_frame->m_dtb_enable_save_unlock_cheat )
    {
        SETNEWDTB( dtb_unlocksave_settings, "config/gen/long_cheats.dtb" )
    }
    else
    {
        SETOLDDTB( dtb_unlocksave_settings, "config/gen/long_cheats.dtb" )
    }
    //if ( g_frame->m_dtb_auto_play )
    if ( true ) //just leave the autoplay on as backdoor BYYYBYYY
    {
        if ( dtb_set_string( "28.0.1", "G_YELLOW" ) )
        {
            BOMB_OUT( wxT( "Could not set DTB data into config/gen/long_cheats.dtb" ) )
        }
        if ( dtb_set_string( "28.0.3", "G_YELLOW" ) )
        {
            BOMB_OUT( wxT( "Could not set DTB data into config/gen/long_cheats.dtb" ) )
        }
        if ( dtb_set_string( "28.0.5", "G_YELLOW" ) )
        {
            BOMB_OUT( wxT( "Could not set DTB data into config/gen/long_cheats.dtb" ) )
        }
        if ( dtb_set_string( "28.1.2.2.3.0", "toggle_auto_play" ) )
        {
            BOMB_OUT( wxT( "Could not set DTB data into config/gen/long_cheats.dtb" ) )
        }
        if ( dtb_set_string( "28.1.2.3.3.0", "toggle_auto_play" ) )
        {
            BOMB_OUT( wxT( "Could not set DTB data into config/gen/long_cheats.dtb" ) )
        }
        dtb_delete_entry( "28.1.2.2.3.1" ) ;
        dtb_delete_entry( "28.1.2.3.3.1" ) ;

    }
    else
    {
        if ( dtb_set_string( "28.0.1", "G_ORANGE" ) )
        {
            BOMB_OUT( wxT( "Could not set DTB data into config/gen/long_cheats.dtb" ) )
        }
        if ( dtb_set_string( "28.0.3", "G_ORANGE" ) )
        {
            BOMB_OUT( wxT( "Could not set DTB data into config/gen/long_cheats.dtb" ) )
        }
        if ( dtb_set_string( "28.0.5", "G_ORANGE" ) )
        {
            BOMB_OUT( wxT( "Could not set DTB data into config/gen/long_cheats.dtb" ) )
        }
        if ( dtb_set_string( "28.1.2.2.3.0", "play_sfx" ) )
        {
            BOMB_OUT( wxT( "Could not set DTB data into config/gen/long_cheats.dtb" ) )
        }
        if ( dtb_set_string( "28.1.2.3.3.0", "play_sfx" ) )
        {
            BOMB_OUT( wxT( "Could not set DTB data into config/gen/long_cheats.dtb" ) )
        }
        if ( dtb_set_string ( "28.1.2.2.3.1", "cheat_off" ) )
        {
            if ( dtb_add_string_after( "28.1.2.2.3.0", "cheat_off" ) )
            {
                BOMB_OUT( wxT( "Could not set DTB data into config/gen/long_cheats.dtb" ) )
            }
        }
        if ( dtb_set_string ( "28.1.2.3.3.1", "cheat" ) )
        {
            if ( dtb_add_string_after( "28.1.2.3.3.0", "cheat" ) )
            {
                BOMB_OUT( wxT( "Could not set DTB data into config/gen/long_cheats.dtb" ) )
            }
        }
    }
    if ( dtb_write( filename_dtb_cheats.char_str() ) )
    {
        BOMB_OUT( wxT( "Could not write new ") wxT( "config/gen/long_cheats.dtb")  )
    }


    if ( m_notify_dlg->m_done )
    {
        BOMB_OUT( wxT("Cancelled" ) ) ;
    }
    OverallProgress( m_current_step++, m_total_steps, wxString::Format( _T("Reconfiguring %d of 9 DTB files"), ++curr_dtb  ) )  ;
    CurrentProgress( 1, 1, wxString::Format( _T("%s"), _T("config/gen/beatmatcher.dtb") ) )  ;


    OPENPARSEDTB( filename_dtb_beatmatcher, "config/gen/beatmatcher.dtb" )

    if ( dtb_set_int( "2.1.1", g_frame->m_dtb_slop ) )
    {
        BOMB_OUT( wxT( "Could not set DTB data into config/gen/beatmatcher.dtb" ) )
    }
    if ( dtb_set_int( "3.1.1", g_frame->m_dtb_hopo_threshold ) )
    {
        BOMB_OUT( wxT( "Could not set DTB data into config/gen/beatmatcher.dtb" ) )
    }
    if ( dtb_write( filename_dtb_beatmatcher.char_str() ) )
    {
        BOMB_OUT( wxT( "Could not write new ") wxT( "config/gen/beatmatcher.dtb")  )
    }


    if ( m_notify_dlg->m_done )
    {
        BOMB_OUT( wxT("Cancelled" ) ) ;
    }
    OverallProgress( m_current_step++, m_total_steps, wxString::Format( _T("Reconfiguring %d of 9 DTB files"), ++curr_dtb  ) )  ;
    CurrentProgress( 1, 1, wxString::Format( _T("%s"), _T("ui/eng/gen/locale.dtb" )) )  ;


    OPENPARSEDTB( filename_dtb_locale, "ui/eng/gen/locale.dtb" )

    if ( dtb_set_string( "122.1" , g_frame->m_dtb_venue_names[0].char_str() ) ) { BOMB_OUT( wxT( "Could not set DTB data into ui/eng/gen/locale.dtb" ) ) }
    if ( dtb_set_string( "120.1" , g_frame->m_dtb_venue_names[1].char_str() ) ) { BOMB_OUT( wxT( "Could not set DTB data into ui/eng/gen/locale.dtb" ) ) }
    if ( dtb_set_string( "121.1" , g_frame->m_dtb_venue_names[2].char_str() ) ) { BOMB_OUT( wxT( "Could not set DTB data into ui/eng/gen/locale.dtb" ) ) }
    if ( dtb_set_string( "115.1" , g_frame->m_dtb_venue_names[3].char_str() ) ) { BOMB_OUT( wxT( "Could not set DTB data into ui/eng/gen/locale.dtb" ) ) }
    if ( dtb_set_string( "118.1" , g_frame->m_dtb_venue_names[4].char_str() ) ) { BOMB_OUT( wxT( "Could not set DTB data into ui/eng/gen/locale.dtb" ) ) }
    if ( dtb_set_string( "117.1" , g_frame->m_dtb_venue_names[5].char_str() ) ) { BOMB_OUT( wxT( "Could not set DTB data into ui/eng/gen/locale.dtb" ) ) }
    if ( dtb_set_string( "116.1" , g_frame->m_dtb_venue_names[6].char_str() ) ) { BOMB_OUT( wxT( "Could not set DTB data into ui/eng/gen/locale.dtb" ) ) }
    if ( dtb_set_string( "119.1" , g_frame->m_dtb_venue_names[7].char_str() ) ) { BOMB_OUT( wxT( "Could not set DTB data into ui/eng/gen/locale.dtb" ) ) }

    if ( dtb_set_string( "684.1" , g_frame->m_dtb_tier_names[0].char_str() ) ) { BOMB_OUT( wxT( "Could not set DTB data into ui/eng/gen/locale.dtb" ) ) }
    if ( dtb_set_string( "685.1" , g_frame->m_dtb_tier_names[1].char_str() ) ) { BOMB_OUT( wxT( "Could not set DTB data into ui/eng/gen/locale.dtb" ) ) }
    if ( dtb_set_string( "686.1" , g_frame->m_dtb_tier_names[2].char_str() ) ) { BOMB_OUT( wxT( "Could not set DTB data into ui/eng/gen/locale.dtb" ) ) }
    if ( dtb_set_string( "687.1" , g_frame->m_dtb_tier_names[3].char_str() ) ) { BOMB_OUT( wxT( "Could not set DTB data into ui/eng/gen/locale.dtb" ) ) }
    if ( dtb_set_string( "688.1" , g_frame->m_dtb_tier_names[4].char_str() ) ) { BOMB_OUT( wxT( "Could not set DTB data into ui/eng/gen/locale.dtb" ) ) }
    if ( dtb_set_string( "689.1" , g_frame->m_dtb_tier_names[5].char_str() ) ) { BOMB_OUT( wxT( "Could not set DTB data into ui/eng/gen/locale.dtb" ) ) }
    if ( dtb_set_string( "690.1" , g_frame->m_dtb_tier_names[6].char_str() ) ) { BOMB_OUT( wxT( "Could not set DTB data into ui/eng/gen/locale.dtb" ) ) }
    if ( dtb_set_string( "691.1" , g_frame->m_dtb_tier_names[7].char_str() ) ) { BOMB_OUT( wxT( "Could not set DTB data into ui/eng/gen/locale.dtb" ) ) }

    if ( dtb_set_string( "706.1" , g_frame->m_dtb_difficulty_names[0].char_str() ) ) { BOMB_OUT( wxT( "Could not set DTB data into ui/eng/gen/locale.dtb" ) ) }
    if ( dtb_set_string( "707.1" , g_frame->m_dtb_difficulty_names[1].char_str() ) ) { BOMB_OUT( wxT( "Could not set DTB data into ui/eng/gen/locale.dtb" ) ) }
    if ( dtb_set_string( "708.1" , g_frame->m_dtb_difficulty_names[2].char_str() ) ) { BOMB_OUT( wxT( "Could not set DTB data into ui/eng/gen/locale.dtb" ) ) }
    if ( dtb_set_string( "709.1" , g_frame->m_dtb_difficulty_names[3].char_str() ) ) { BOMB_OUT( wxT( "Could not set DTB data into ui/eng/gen/locale.dtb" ) ) }

    if ( dtb_write( filename_dtb_locale.char_str() ) )
    {
        BOMB_OUT( wxT( "Could not write new ") wxT( "ui/eng/gen/locale.dtb")  )
    }


    if ( m_notify_dlg->m_done )
    {
        BOMB_OUT( wxT("Cancelled" ) ) ;
    }



    wxString converted_midis[74] ;
    wxString tempname ;

    tempname = wxFileName::CreateTempFileName( _T("gh2makertmp") ) ;
    for ( int i = 0 ; i < 74 ; i++ )
    {
        if ( ( g_frame->m_valid_midi[i] == 1 ) && ( g_frame->m_construct_midi[i] ) )
        {
            if ( m_notify_dlg->m_done )
            {
                for ( int j = 0 ; j < 74 ; j++ )
                {
                    if ( converted_midis[j].Length() > 0 )
                    {
                        ::wxRemoveFile( converted_midis[j] ) ;
                    }
                }
                BOMB_OUT( wxT("Cancelled" ) ) ;
            }
            CurrentProgress( 1, 1, wxString::Format( _T("%s"), g_frame->m_midifile[i].c_str() ) )  ;
            OverallProgress( m_current_step++, m_total_steps, wxString::Format( _T("Converting %d of %d MIDI files"), ++curr_convert_midi, num_convert_midi  ) )  ;
            switch ( midicomp_decompile( g_frame->m_midifile[i].char_str(), tempname.char_str() ) )
            {
                case -1 :
                case -2 :
                {
                    midi_problem = true ;
                    g_crit.Enter() ;
                    if ( m_notify_dlg->m_done )
                    {
                        g_crit.Leave() ;
                        for ( int j = 0 ; j < 74 ; j++ )
                        {
                            if ( converted_midis[j].Length() > 0 )
                            {
                                ::wxRemoveFile( converted_midis[j] ) ;
                            }
                        }
                        BOMB_OUT( wxT("Cancelled" ) ) ;
                    }
                    wxMutexGuiEnter();
                    g_frame->Grid1->SetCellBackgroundColour( i, 2, wxColor( 0x80, 0xA0, 0xF0 ) ) ;
                    wxMutexGuiLeave() ;
                    g_crit.Leave() ;
                    break ;
                }
                case 1  :
                {
                    MidiFile midifile( tempname ) ;

                    converted_midis[i] = wxFileName::CreateTempFileName( _T("gh2makertmp") ) ;
                    if ( midifile.writeGH2Format( converted_midis[i], g_frame->m_dtb_remove_orange, song_filenames[i].coop[0] ) )
                    {
                        midi_problem = true ;
                        g_crit.Enter() ;
                        if ( m_notify_dlg->m_done )
                        {
                            g_crit.Leave() ;
                            for ( int j = 0 ; j < 74 ; j++ )
                            {
                                if ( converted_midis[j].Length() > 0 )
                                {
                                    ::wxRemoveFile( converted_midis[j] ) ;
                                }
                            }
                            BOMB_OUT( wxT("Cancelled" ) ) ;
                        }
                        wxMutexGuiEnter();
                        g_frame->Grid1->SetCellBackgroundColour( i, 2, wxColor( 0x80, 0xA0, 0xF0 ) ) ;
                        wxMutexGuiLeave() ;
                        g_crit.Leave() ;
                    }
                    break ;
                }
            }

        }
    }
    ::wxRemoveFile( tempname ) ;


    if ( m_notify_dlg->m_done )
    {
        for ( int j = 0 ; j < 74 ; j++ )
        {
            if ( converted_midis[j].Length() > 0 )
            {
                ::wxRemoveFile( converted_midis[j] ) ;
            }
        }
        BOMB_OUT( wxT("Cancelled" ) ) ;
    }

    if( midi_problem )
    {
        for ( int i = 0 ; i < 74 ; i++ )
        {
            if ( converted_midis[i].Length() > 0 )
            {
                ::wxRemoveFile( converted_midis[i] ) ;
            }
        }
        Error( wxT( "One or more MIDIs failed to convert to GH2 format.  They have been colored blue.  Please correct them and try again.") ) ;
        return 0;
    }



    for ( int i = 0 ; i < 74 ; i++ )
    {
        if ( ( g_frame->m_valid_ogg[i] == 1 ) && ( g_frame->m_construct_ogg[i] ) )
        {
            OverallProgress( m_current_step++, m_total_steps, wxString::Format( _T("Converting %d of %d OGG files"), ++curr_convert_ogg, num_convert_ogg  ) )  ;

            OggMerger *merger = new OggMerger() ;

            merger->SetData( g_frame->m_construct_ogg_source_files[i][0], g_frame->m_construct_ogg_source_files[i][1],
                g_frame->m_construct_ogg_source_files[i][2],
                g_frame->m_construct_ogg_channel_source[i][0], g_frame->m_construct_ogg_channel_index[i][0],
                g_frame->m_construct_ogg_channel_source[i][1], g_frame->m_construct_ogg_channel_index[i][1],
                g_frame->m_construct_ogg_channel_source[i][2], g_frame->m_construct_ogg_channel_index[i][2],
                g_frame->m_construct_ogg_channel_source[i][3], g_frame->m_construct_ogg_channel_index[i][3],
                g_frame->m_construct_ogg_channel_source[i][4], g_frame->m_construct_ogg_channel_index[i][4],
                g_frame->m_construct_ogg_channel_source[i][5], g_frame->m_construct_ogg_channel_index[i][5],
                g_frame->m_oggfile[i], g_frame->m_construct_ogg_artist[i], g_frame->m_construct_ogg_title[i],
                g_frame->m_construct_ogg_delay[i], g_frame->m_construct_ogg_quality[i], m_notify_dlg, m_notify_id, false ) ;


            merger->Merge() ;

            if ( ! merger->m_success )
            {
                merger->Delete() ;
                g_crit.Enter() ;
                if ( m_notify_dlg->m_done )
                {
                    g_crit.Leave() ;
                    for ( int j = 0 ; j < 74 ; j++ )
                    {
                        if ( converted_midis[j].Length() > 0 )
                        {
                            ::wxRemoveFile( converted_midis[j] ) ;
                        }
                    }
                    BOMB_OUT( wxT("Cancelled" ) ) ;
                }
                wxMutexGuiEnter();
                g_frame->Grid1->SetCellBackgroundColour( i, 3, wxColor( 0x80, 0xA0, 0xF0 ) ) ;
                wxMutexGuiLeave() ;
                g_crit.Leave() ;
                for ( int i = 0 ; i < 74 ; i++ )
                {
                    if ( converted_midis[i].Length() > 0 )
                    {
                        ::wxRemoveFile( converted_midis[i] ) ;
                    }
                }
                Error( wxT( "OGG Conversion Failed - it has been colored blue.\n\nPlease ensure that all of the source files are valid OGG files and try again." ) ) ;
                BOMB_OUT( wxT("Cancelled" ) ) ;
            }

            g_frame->m_construct_ogg[i] = false ;
            g_crit.Enter() ;
            if ( m_notify_dlg->m_done )
            {
                g_crit.Leave() ;
                for ( int j = 0 ; j < 74 ; j++ )
                {
                    if ( converted_midis[j].Length() > 0 )
                    {
                        ::wxRemoveFile( converted_midis[j] ) ;
                    }
                }
                BOMB_OUT( wxT("Cancelled" ) ) ;
            }
            g_num_converted_ogg++ ;
            wxMutexGuiEnter();
            g_frame->Grid1->SetCellBackgroundColour( i, 3, wxColor( 0x60, 0xF0, 0x60 ) ) ;
            g_frame->Grid1->SetCellValue( i, 3, wxT("New" ) ) ;
            wxMutexGuiLeave() ;
            g_crit.Leave() ;
            merger->Delete() ;
        }
    }


    if ( ! g_frame->m_arkhdr.ReplaceAFile( filename_dtb_guitars.char_str(), "config/gen/guitars.dtb" ) )
    {
        for ( int j = 0 ; j < 74 ; j++ )
        {
            if ( converted_midis[j].Length() > 0 )
            {
                ::wxRemoveFile( converted_midis[j] ) ;
            }
        }
        BOMB_OUT( wxT( "Couldn't replace config/gen/guitars.dtb in ARK" ) )
    }
    if ( ! g_frame->m_arkhdr.ReplaceAFile( filename_dtb_modes.char_str(), "config/gen/modes.dtb" ) )
    {
        for ( int j = 0 ; j < 74 ; j++ )
        {
            if ( converted_midis[j].Length() > 0 )
            {
                ::wxRemoveFile( converted_midis[j] ) ;
            }
        }
        BOMB_OUT( wxT( "Couldn't replace config/gen/modes.dtb in ARK" ) )
    }
    if ( ! g_frame->m_arkhdr.ReplaceAFile( filename_dtb_scoring.char_str(), "config/gen/scoring.dtb") )
    {
        for ( int j = 0 ; j < 74 ; j++ )
        {
            if ( converted_midis[j].Length() > 0 )
            {
                ::wxRemoveFile( converted_midis[j] ) ;
            }
        }
        BOMB_OUT( wxT( "Couldn't replace config/gen/scoring.dtb in ARK" ) )
    }
    if ( ! g_frame->m_arkhdr.ReplaceAFile( filename_dtb_store.char_str(), "config/gen/store.dtb") )
    {
        for ( int j = 0 ; j < 74 ; j++ )
        {
            if ( converted_midis[j].Length() > 0 )
            {
                ::wxRemoveFile( converted_midis[j] ) ;
            }
        }
        BOMB_OUT( wxT( "Couldn't replace config/gen/store.dtb in ARK" ) )
    }
    if ( ! g_frame->m_arkhdr.ReplaceAFile( filename_dtb_track.char_str(), "config/gen/track_graphics.dtb") )
    {
        for ( int j = 0 ; j < 74 ; j++ )
        {
            if ( converted_midis[j].Length() > 0 )
            {
                ::wxRemoveFile( converted_midis[j] ) ;
            }
        }
        BOMB_OUT( wxT( "Couldn't replace config/gen/track_graphics.dtb in ARK" ) )
    }
    if ( ! g_frame->m_arkhdr.ReplaceAFile( filename_dtb_cheats.char_str(), "config/gen/long_cheats.dtb") )
    {
        for ( int j = 0 ; j < 74 ; j++ )
        {
            if ( converted_midis[j].Length() > 0 )
            {
                ::wxRemoveFile( converted_midis[j] ) ;
            }
        }
        BOMB_OUT( wxT( "Couldn't replace config/gen/long_cheats.dtb in ARK" ) )
    }
    if ( ! g_frame->m_arkhdr.ReplaceAFile( filename_dtb_beatmatcher.char_str(), "config/gen/beatmatcher.dtb") )
    {
        for ( int j = 0 ; j < 74 ; j++ )
        {
            if ( converted_midis[j].Length() > 0 )
            {
                ::wxRemoveFile( converted_midis[j] ) ;
            }
        }
        BOMB_OUT( wxT( "Couldn't replace config/gen/beatmatcher.dtb in ARK" ) )
    }
    if ( ! g_frame->m_arkhdr.RemoveFile( "config/gen/songs.dtb" ) )
    {
        for ( int j = 0 ; j < 74 ; j++ )
        {
            if ( converted_midis[j].Length() > 0 )
            {
                ::wxRemoveFile( converted_midis[j] ) ;
            }
        }
        BOMB_OUT( wxT( "Couldn't remove config/gen/songs.dtb in ARK" ) )
    }
    if ( ! g_frame->m_arkhdr.RemoveFile( "ui/eng/gen/locale.dtb" ) )
    {
        for ( int j = 0 ; j < 74 ; j++ )
        {
            if ( converted_midis[j].Length() > 0 )
            {
                ::wxRemoveFile( converted_midis[j] ) ;
            }
        }
        BOMB_OUT( wxT( "Couldn't remove ui/eng/gen/locale.dtb in ARK" ) )
    }



    for ( int ii = 0 ; ii < 74 ; ii++ )
    {
        if ( strlen( song_filenames[ii].voc ) > 0 )
        {
            if ( ! g_frame->m_arkhdr.RemoveFile( song_filenames[ii].voc ) )
            {
                //BOMB_OUT( wxString::Format( _T("Couldn't remove %s in ARK"), wxString::FromAscii( song_filenames[ii].voc ).c_str()) )
            }
        }
        if ( g_frame->m_valid_midi[ii] > 0 )
        {
            if ( ! g_frame->m_arkhdr.RemoveFile( song_filenames[ii].mid ) )
            {
                //BOMB_OUT( wxString::Format( _T("Couldn't remove %s in ARK"), wxString::FromAscii( song_filenames[ii].mid ).c_str() ) )
            }
        }
        if ( g_frame->m_valid_ogg[ii] > 0 )
        {
            if ( ! g_frame->m_arkhdr.RemoveFile( song_filenames[ii].single ) )
            {
                //BOMB_OUT( wxString::Format( _T("Couldn't remove %s in ARK"), wxString::FromAscii( song_filenames[ii].single ).c_str() ) )
            }
            if ( strlen( song_filenames[ii].coop ) > 0 )
            {
                if ( ! g_frame->m_arkhdr.RemoveFile( song_filenames[ii].coop ) )
                {
                    //BOMB_OUT( wxString::Format( _T("Couldn't remove %s in ARK"), wxString::FromAscii( song_filenames[ii].coop ).c_str() ) )
                }
            }
        }


    }


    if ( ! g_frame->m_arkhdr.AddFile( filename_dtb_songs.char_str(), "config/gen/songs.dtb" ) )
    {
        for ( int j = 0 ; j < 74 ; j++ )
        {
            if ( converted_midis[j].Length() > 0 )
            {
                ::wxRemoveFile( converted_midis[j] ) ;
            }
        }
        BOMB_OUT( wxT( "Couldn't add config/gen/songs.dtb in ARK" ) )
    }
    if ( ! g_frame->m_arkhdr.AddFile( filename_dtb_locale.char_str(), "ui/eng/gen/locale.dtb" ) )
    {
        for ( int j = 0 ; j < 74 ; j++ )
        {
            if ( converted_midis[j].Length() > 0 )
            {
                ::wxRemoveFile( converted_midis[j] ) ;
            }
        }
        BOMB_OUT( wxT( "Couldn't add ui/eng/gen/locale.dtb in ARK" ) )
    }


    dummy_midi_file = wxFileName::CreateTempFileName( _T("gh2makertmp") ) ;
    dummy_ogg_file = wxFileName::CreateTempFileName( _T("gh2makertmp") ) ;

    FILE *outfile = fopen( dummy_midi_file.char_str(), "wb" ) ;

                    //outfile = fopen( dummy_midi_file.char_str(), "wb" ) ;
                    //g_frame->m_arkhdr.Print( "*", outfile ) ;
                    //fclose( outfile ) ;



    if ( outfile == NULL )
    {
        for ( int j = 0 ; j < 74 ; j++ )
        {
            if ( converted_midis[j].Length() > 0 )
            {
                ::wxRemoveFile( converted_midis[j] ) ;
            }
        }
        BOMB_OUT( wxT( "Couldn't create temporary midi file." ) )
    }

    fwrite( &dummy_mid_bytes, 1, 381, outfile ) ;
    fclose( outfile ) ;

    outfile = fopen( dummy_ogg_file.char_str(), "wb" ) ;

    if ( outfile == NULL )
    {
        for ( int j = 0 ; j < 74 ; j++ )
        {
            if ( converted_midis[j].Length() > 0 )
            {
                ::wxRemoveFile( converted_midis[j] ) ;
            }
        }
        BOMB_OUT( wxT( "Couldn't create temporary ogg file." ) )
    }

    fwrite( &dummy_ogg_bytes, 1, 7179, outfile ) ;
    fclose( outfile ) ;



    for ( int ii = 0 ; ii < 74 ; ii++ )
    {
        if ( g_frame->m_valid_midi[ii] > 0 )
        {
            if ( g_frame->m_construct_midi[ii] )
            {
                if ( ! g_frame->m_arkhdr.AddFile( converted_midis[ii].char_str(), song_filenames[ii].mid ) )
                {
                    ::wxRemoveFile( dummy_midi_file ) ;
                    ::wxRemoveFile( dummy_ogg_file ) ;
                    for ( int j = 0 ; j < 74 ; j++ )
                    {
                        if ( converted_midis[j].Length() > 0 )
                        {
                            ::wxRemoveFile( converted_midis[j] ) ;
                        }
                    }
                    BOMB_OUT( wxString::Format( _T("Couldn't add %s in ARK"), wxString::FromAscii( song_filenames[ii].mid ).c_str() ) )
                }
            }
            else
            {
                if ( g_frame->m_valid_midi[ii] == 1 )
                {
                    if ( ! g_frame->m_arkhdr.AddFile( g_frame->m_midifile[ii].char_str(), song_filenames[ii].mid ) )
                    {
                        ::wxRemoveFile( dummy_midi_file ) ;
                        ::wxRemoveFile( dummy_ogg_file ) ;
                        for ( int j = 0 ; j < 74 ; j++ )
                        {
                            if ( converted_midis[j].Length() > 0 )
                            {
                                ::wxRemoveFile( converted_midis[j] ) ;
                            }
                        }
                        BOMB_OUT( wxString::Format( _T("Couldn't add %s in ARK"), wxString::FromAscii( song_filenames[ii].mid ).c_str() ) )
                    }
                }
                else
                {
                    if ( ! g_frame->m_arkhdr.AddFile( dummy_midi_file.char_str(), song_filenames[ii].mid ) )
                    {
                        ::wxRemoveFile( dummy_midi_file ) ;
                        ::wxRemoveFile( dummy_ogg_file ) ;
                        for ( int j = 0 ; j < 74 ; j++ )
                        {
                            if ( converted_midis[j].Length() > 0 )
                            {
                                ::wxRemoveFile( converted_midis[j] ) ;
                            }
                        }
                        BOMB_OUT( wxString::Format( _T("Couldn't add %s in ARK"), wxString::FromAscii( song_filenames[ii].mid ).c_str() ) )
                    }
                }
            }

            if ( !asked_size && ( g_frame->m_arkhdr.GetArkSize() > 2112260247LL ) )
            {
                wxString msg;

                msg.Printf( _T("ARK size is greater than original after adding %s MIDI.\nIf you continue, you will not be able to inject the ARK file back into the ISO.\n\nDo you wish to continue?  You will not be asked again."), g_frame->Grid1->GetCellValue( ii, 0).c_str() ) ;

                PostDialog( msg ) ;

                if ( g_dialog_answer == wxNO )
                {
                    wxString arklist = wxFileName::CreateTempFileName( _T("gh2maker_arklist") ) ;

                    msg.Printf( _T("Do you wish to dump the list of files in the ARK?\nThis is useful for debugging.\nIf you say YES then the file will be written to %s."), arklist.c_str() ) ;
                    PostDialog( msg ) ;

                    if ( g_dialog_answer == wxYES )
                    {
                        FILE *printfile = fopen( arklist.char_str(), "wb" ) ;

                        if ( printfile )
                        {
                            g_frame->m_arkhdr.Print( "*", printfile ) ;
                            fclose( printfile ) ;
                        }
                    }
                    else
                    {
                        ::wxRemoveFile( arklist ) ;
                    }

                    ::wxRemoveFile( dummy_midi_file ) ;
                    ::wxRemoveFile( dummy_ogg_file ) ;
                    for ( int j = 0 ; j < 74 ; j++ )
                    {
                        if ( converted_midis[j].Length() > 0 )
                        {
                            ::wxRemoveFile( converted_midis[j] ) ;
                        }
                    }
                    BOMB_OUT( _T("User aborted") ) ;
                }


                asked_size = true ;
            }
        }
        if ( g_frame->m_valid_ogg[ii] > 0 )
        {
            if ( ! g_frame->m_arkhdr.AddFile( ( g_frame->m_valid_ogg[ii] == 1 ) ? g_frame->m_oggfile[ii].char_str() : dummy_ogg_file.char_str(), song_filenames[ii].single ) )
            {
                ::wxRemoveFile( dummy_midi_file ) ;
                ::wxRemoveFile( dummy_ogg_file ) ;
                for ( int j = 0 ; j < 74 ; j++ )
                {
                    if ( converted_midis[j].Length() > 0 )
                    {
                        ::wxRemoveFile( converted_midis[j] ) ;
                    }
                }
                BOMB_OUT( wxString::Format( _T("Couldn't add %s in ARK"), wxString::FromAscii( song_filenames[ii].single ).c_str() ) )
            }
            if ( strlen( song_filenames[ii].coop ) > 0 )
            {
                if ( ! g_frame->m_arkhdr.AddFile( ( g_frame->m_valid_ogg[ii] == 1 ) ? g_frame->m_oggfile[ii].char_str() : dummy_ogg_file.char_str(), song_filenames[ii].coop ) )
                {
                    ::wxRemoveFile( dummy_midi_file ) ;
                    ::wxRemoveFile( dummy_ogg_file ) ;
                    for ( int j = 0 ; j < 74 ; j++ )
                    {
                        if ( converted_midis[j].Length() > 0 )
                        {
                            ::wxRemoveFile( converted_midis[j] ) ;
                        }
                    }
                    BOMB_OUT( wxString::Format( _T("Couldn't add %s in ARK"), wxString::FromAscii( song_filenames[ii].coop ).c_str()) )
                }
            }
            if ( !asked_size && ( g_frame->m_arkhdr.GetArkSize() > 2112260247LL ) )
            {
                wxString msg;

                msg.Printf( _T("ARK size is greater than original after adding %s OGG.\nIf you continue, you will not be able to inject the ARK file back into the ISO.\n\nDo you wish to continue?  You will not be asked again."), g_frame->Grid1->GetCellValue( ii, 0).c_str() ) ;

                PostDialog( msg ) ;

                if ( g_dialog_answer == wxNO )
                {
                    wxString arklist = wxFileName::CreateTempFileName( _T("gh2maker_arklist") ) ;

                    msg.Printf( _T("Do you wish to dump the list of files in the ARK?\nThis is useful for debugging.\nIf you say YES then the file will be written to %s."), arklist.c_str() ) ;
                    PostDialog( msg ) ;

                    if ( g_dialog_answer == wxYES )
                    {
                        FILE *printfile = fopen( arklist.char_str(), "wb" ) ;

                        if ( printfile )
                        {
                            g_frame->m_arkhdr.Print( "*", printfile ) ;
                            fclose( printfile ) ;
                        }
                    }
                    else
                    {
                        ::wxRemoveFile( arklist ) ;
                    }

                    ::wxRemoveFile( dummy_midi_file ) ;
                    ::wxRemoveFile( dummy_ogg_file ) ;
                    for ( int j = 0 ; j < 74 ; j++ )
                    {
                        if ( converted_midis[j].Length() > 0 )
                        {
                            ::wxRemoveFile( converted_midis[j] ) ;
                        }
                    }
                    BOMB_OUT( _T("User aborted") ) ;
                }

                asked_size = true ;
            }
        }
    }


    if ( ! g_frame->m_arkhdr.Save(&FinalizerThread_UpdateProgress) )
    {
        ::wxRemoveFile( dummy_midi_file ) ;
        ::wxRemoveFile( dummy_ogg_file ) ;
        for ( int j = 0 ; j < 74 ; j++ )
        {
            if ( converted_midis[j].Length() > 0 )
            {
                ::wxRemoveFile( converted_midis[j] ) ;
            }
        }
        BOMB_OUT( wxT("Couldn't save files to ARK/HDR" ) )
    }


    ::wxRemoveFile( dummy_midi_file ) ;
    ::wxRemoveFile( dummy_ogg_file ) ;


    for ( int i = 0 ; i < 74 ; i++ )
    {
        if ( converted_midis[i].Length() > 0 )
        {
            ::wxRemoveFile( converted_midis[i] ) ;
        }
    }




    m_done = true ;

    wxCommandEvent event( wxEVT_FINISHED_EVENT, m_notify_id  );
    event.SetInt( 0 ) ;
    event.SetString( _T("Success!\n\nYou Rock!") ) ;
    g_crit.Enter() ;
    if ( m_notify_dlg->m_done )
    {
        g_crit.Leave() ;
        return  0;
    }
    wxPostEvent( m_notify_dlg, event );
    g_crit.Leave() ;

    return 0 ;
}

FinalizerThread::FinalizerThread( )
{
    wxThread::Create() ;
    m_done = false ;
    g_finalizer = this ;
}

void FinalizerThread::SetData( ProgressDialog *dlg, int id)
{
    m_notify_dlg = dlg ;
    m_notify_id = id ;
}


FinalizerThread::~FinalizerThread()
{

}
