#include <wx/wxprec.h>

#ifdef __BORLANDC__
        #pragma hdrstop
#endif

#ifndef WX_PRECOMP
        #include <wx/wx.h>
#endif

#include <wx/txtstrm.h>
#include <wx/wfstream.h>
#include <wx/textfile.h>
#include <wx/arrstr.h>
#include <wx/tokenzr.h>
#include <wx/filename.h>
#include "wxArrayStringArray.h"


#include "MidiFile.h"


#ifdef __cplusplus
extern "C" {
#endif

int midicomp_compile( char *ifilename, char *ofilename ) ;

#ifdef __cplusplus
}
#endif



MidiFile::MidiFile (wxString filename)
{
    wxString line ;
    int count_easy, count_medium, count_hard, count_expert ;
    wxString tmpstr ;
    wxFFileInputStream fis( filename ) ;
    wxTextInputStream tis( fis ) ;
    wxArrayString blanktrack ;
    int  currtrack ;
    long  currseq ;
    long  lasttempo ;

    blanktrack.Empty() ;
    m_track_names.clear() ;
    m_tempo_lines.clear() ;
    currtrack = -1 ;
    m_maxseq = 0 ;
    m_is_gh1 = false ;
    lasttempo = 0 ;
    m_guitar_track = -1 ;
    m_fallback_guitar_track = -1 ;
    m_guitar_coop_track = -1 ;
    m_guitar_bass_track  = -1 ;
    m_band_singer_track  = -1 ;
    m_band_drums_track  = -1 ;
    m_band_bass_track  = -1 ;
    m_events_track  = -1 ;
    m_triggers_track  = -1 ;
    m_tempo_track = -1 ;
    m_found_tempo = false ;
    m_found_timesig = false ;
    m_has_easy = m_has_expert = m_has_hard = m_has_medium = false ;

    count_easy = count_expert = count_medium = count_hard = 0 ;

    m_header = tis.ReadLine() ;

    while (!fis.Eof())
    {
        line = tis.ReadLine();

        if ( line.Length() == 0 )
            continue ;

        if
        (
            ( !m_has_easy ) &&
            (
                ( line.Contains( _T("n=60") ) ) ||
                ( line.Contains( _T("n=61") ) ) ||
                ( line.Contains( _T("n=62") ) ) ||
                ( line.Contains( _T("n=63") ) ) ||
                ( line.Contains( _T("n=64") ) )
            )
        )
        {
            if ( ++count_easy > 20 )
            {
                m_has_easy = true ;
                if ( m_fallback_guitar_track < 0 )
                    m_fallback_guitar_track = currtrack ;
            }
        }
        if
        (
            ( !m_has_medium ) &&
            (
                ( line.Contains( _T("n=72") ) ) ||
                ( line.Contains( _T("n=73") ) ) ||
                ( line.Contains( _T("n=74") ) ) ||
                ( line.Contains( _T("n=75") ) ) ||
                ( line.Contains( _T("n=76") ) )
            )
        )
        {
            if ( ++count_medium > 20 )
            {
                m_has_medium = true ;
                if ( m_fallback_guitar_track < 0 )
                    m_fallback_guitar_track = currtrack ;
            }
        }
        if
        (
            ( !m_has_hard ) &&
            (
                ( line.Contains( _T("n=84") ) ) ||
                ( line.Contains( _T("n=85") ) ) ||
                ( line.Contains( _T("n=86") ) ) ||
                ( line.Contains( _T("n=87") ) ) ||
                ( line.Contains( _T("n=88") ) )
            )
        )
        {
            if ( ++count_hard > 20 )
            {
                m_has_hard = true ;
                if ( m_fallback_guitar_track < 0 )
                    m_fallback_guitar_track = currtrack ;
            }
        }
        if
        (
            ( !m_has_expert ) &&
            (
                ( line.Contains( _T("n=96") ) ) ||
                ( line.Contains( _T("n=97") ) ) ||
                ( line.Contains( _T("n=98") ) ) ||
                ( line.Contains( _T("n=99") ) ) ||
                ( line.Contains( _T("n=100") ) )
            )
        )
        {
            if ( ++count_expert > 20 )
            {
                m_has_expert = true ;
                if ( m_fallback_guitar_track < 0 )
                    m_fallback_guitar_track = currtrack ;
            }
        }




        if ( ( line.BeforeFirst( ' ' ).ToLong( &currseq ) ) &&  ( currseq > m_maxseq ) )
                m_maxseq = currseq ;

        if ( line.IsSameAs( _T("MTrk") ) )
        {
            /*
            if ( currtrack >= 0 )
            {
                if ( m_tracks[currtrack].GetCount() == 0 )
                {
                    m_track_names[currtrack] = _T("") ;
                    continue ;
                }
            }
*/
            currtrack++ ;


            m_track_names.Add( _T("") ) ;
            m_tracks.Add( blanktrack ) ;
            continue ;

        }
        else if ( line.Contains( _T( "SeqName" ) ) )
        {
            if ( currtrack == 0 )
                m_tempo_track = 0 ;
            continue ;
        }
        else if ( line.Contains( _T( "TrkName" ) ) )
        {
            tmpstr = line.AfterFirst( '"' ).BeforeFirst( '"' ) ;

            //wxMessageBox( line ) ;
            if ( tmpstr.IsSameAs( _T( "T1 GEMS" ) ) )
            {
                m_is_gh1 = true ;
                tmpstr = _T( "PART GUITAR" ) ;
                line = _T( "0 Meta TrkName \"PART GUITAR\"" ) ;
                m_guitar_track = currtrack ;
            }
            else if ( tmpstr.IsSameAs( _T( "PART GUITAR" ) ) )
                m_guitar_track = currtrack ;
            else if ( tmpstr.IsSameAs( _T( "PART GUITAR COOP" ) ) )
                m_guitar_coop_track = currtrack ;
            else if ( ( tmpstr.IsSameAs( _T( "PART BASS" ) ) ) || ( tmpstr.IsSameAs( _T( "PART RHYTHM" ) ) ) )
                m_guitar_bass_track = currtrack ;
            else if ( tmpstr.IsSameAs( _T( "BAND SINGER" ) ) )
                m_band_singer_track = currtrack ;
            else if ( tmpstr.IsSameAs( _T( "BAND DRUMS" ) ) )
                m_band_drums_track = currtrack ;
            else if ( tmpstr.IsSameAs( _T( "BAND BASS" ) ) )
                m_band_bass_track = currtrack ;
            else if ( tmpstr.IsSameAs( _T( "EVENTS" ) ) )
                m_events_track = currtrack ;
            else if ( tmpstr.IsSameAs( _T( "TRIGGERS" ) ) )
                m_triggers_track = currtrack ;

            m_track_names[currtrack] = tmpstr ;

            continue ;
        }
        else if ( ( line.Contains( _T( "Tempo" ) ) ) || ( line.Contains( _T( "TimeSig" ) ) ) )
        {
            if ( currtrack == 0 )
            {
                if ( line.Contains( _T( "Tempo" ) ) )
                    m_found_tempo = true ;
                if ( line.Contains( _T( "TimeSig" ) ) )
                    m_found_timesig = true ;
            }

            if ( ( line.BeforeFirst( ' ' ).ToLong( &currseq ) ) &&  ( currseq >= lasttempo ) )
            {
                m_tempo_lines.Add( line ) ;
            }
            else
            {
                currseq = 999999999 ;
            }

            lasttempo = currseq ;
        }
        else if ( line.Contains( _T( "Meta TrkEnd" ) ) )
        {
            continue ;
        }
        else if ( line.IsSameAs( _T("TrkEnd") ) )
        {
            continue ;
        }
        else if ( line.Contains( _T( " Par " ) ) )
        {
            continue ;
        }
        else if ( line.Contains( _T( " PrCh " ) ) )
        {
            continue ;
        }
        else if ( line.Contains( _T( " Pb " ) ) )
        {
            continue ;
        }
        else if ( line.Contains( _T( " SeqSpec " ) ) )
        {
            continue ;
        }

        if ( currtrack >= 0 )
            m_tracks[currtrack].Add( line ) ;
    }



}

MidiFile::~MidiFile()
{

}

void MidiFile::PrintNoteTrack( FILE *outfile, int trackno, char *trackname, int remove_orange  )
{
    long bluestate_easy, bluestate_med, bluestate_hard, bluestate_expert ;
    long notevalue = 0 ;
    long note_on = 0 ;
    wxString line = _T("0 ") ;

    bluestate_easy = bluestate_expert = bluestate_hard = bluestate_med = 0 ;

	fprintf( outfile, "MTrk\n" ) ;
	fprintf( outfile, "0 Meta TrkName \"%s\"\n", trackname ) ;

    if ( !m_has_easy )
    {
        fprintf( outfile, "0 On ch=1 n=60 v=100\n" ) ;
    }
    if ( !m_has_medium )
    {
        fprintf( outfile, "0 On ch=1 n=72 v=100\n" ) ;
    }
    if ( !m_has_hard )
    {
        fprintf( outfile, "0 On ch=1 n=84 v=100\n" ) ;
    }
    if ( !m_has_expert )
    {
        fprintf( outfile, "0 On ch=1 n=96 v=100\n" ) ;
    }


    for  ( unsigned int i = 0 ; i < m_tracks[trackno].GetCount() ; i++ )
    {
        line = m_tracks[trackno][i] ;


        if ( ! ( ( line.Contains( _T( "TimeSig" ) ) || line.Contains( _T( "Tempo" ) ) ) ) )
        {
            if ( remove_orange )
            {
                if ( line.Contains( _T( "n=" ) ) )
                {
                    line.AfterFirst( '=' ).AfterFirst( '=' ).BeforeFirst( ' ').ToLong( &notevalue ) ;

                    if ( ( notevalue < 60 ) || ( notevalue > 108 ) )
                    {
                        continue ;
                    }
                    if ( line.Contains( _T("n=64") ) )
                    {
                        line.Replace( _T( "n=64" ), _T( "n=63" ) ) ;
                    }
                    if ( line.Contains( _T("n=76") ) )
                    {
                        line.Replace( _T( "n=76" ), _T( "n=75" ) ) ;
                    }
                    if ( line.Contains( _T("n=88") ) )
                    {
                        line.Replace( _T( "n=88" ), _T( "n=87" ) ) ;
                    }
                    if ( line.Contains( _T("n=100") ) )
                    {
                        line.Replace( _T( "n=100" ), _T( "n=99" ) ) ;
                    }

                    /*
                    if ( ( line.Contains( _T( "Off" ) ) ) || ( line.Contains( _T( "v=0" ) ) ) )
                    {
                        note_on = 0 ;
                    }
                    else
                    {
                        note_on = 1 ;
                    }

                    switch ( notevalue )
                    {
                        case 63 : bluestate_easy = note_on ; break ;
                        case 75 : bluestate_med = note_on ; break ;
                        case 87 : bluestate_hard = note_on ; break ;
                        case 99 : bluestate_expert = note_on ; break ;
                        case 64 :
                    }
                    */


                }



            }

            fprintf( outfile, "%s\n", (const char*)line.char_str() ) ;;


        }
    }

    if ( !m_has_easy )
    {
        fprintf( outfile, "%s On ch=1 n=60 v=0\n", (const char*)line.BeforeFirst( ' ' ).char_str() ) ;
    }
    if ( !m_has_medium )
    {
        fprintf( outfile, "%s On ch=1 n=72 v=0\n", (const char*)line.BeforeFirst( ' ' ).char_str() ) ;
    }
    if ( !m_has_hard )
    {
        fprintf( outfile, "%s On ch=1 n=84 v=0\n", (const char*)line.BeforeFirst( ' ' ).char_str() ) ;
    }
    if ( !m_has_expert )
    {
        fprintf( outfile, "%s On ch=1 n=96 v=0\n", (const char*)line.BeforeFirst( ' ' ).char_str() ) ;
    }


    fprintf( outfile, "%s Meta TrkEnd\n", (const char*)line.BeforeFirst( ' ' ).char_str() ) ;
    fprintf( outfile, "TrkEnd\n" ) ;
}

char *g_midinotes[4][8] =
{
    { "n=60", "n=61", "n=62", "n=63", "n=64",  "n=67",  "n=69",  "n=70" },
    { "n=72", "n=73", "n=74", "n=75", "n=76",  "n=79",  "n=81",  "n=82" },
    { "n=84", "n=85", "n=86", "n=87", "n=88",  "n=91",  "n=93",  "n=94" },
    { "n=96", "n=97", "n=98", "n=99", "n=100", "n=103", "n=105", "n=106" },
};

void MidiFile::Replicate( int to, int from )
{
    int *tracks[3] ;
    int numtracks = 0 ;
    int numlines = 0 ;
    wxArrayString newtrack ;


    if ( m_guitar_track > -1 )
    {
        tracks[numtracks++] = &m_guitar_track ;
    }
    if ( m_guitar_coop_track > -1 )
    {
        tracks[numtracks++] = &m_guitar_coop_track ;
    }
    if ( m_guitar_bass_track > -1 )
    {
        tracks[numtracks++] = &m_guitar_bass_track ;
    }

    for ( int i = 0 ; i < numtracks ; i++ )
    {
        newtrack.Empty() ;

        numlines = m_tracks[*(tracks[i])].GetCount() ;

        for ( int j = 0 ; j < numlines ; j++ )
        {
            wxString line = m_tracks[*(tracks[i])][j] ;

            newtrack.Add( line ) ;
            for ( int k = 0 ; k < 8 ; k++ )
            {
                if ( line.Contains( wxString::FromAscii( g_midinotes[from][k] ) ) )
                {
                    line.Replace( wxString::FromAscii( g_midinotes[from][k] ), wxString::FromAscii( g_midinotes[to][k] ) ) ;
                    newtrack.Add( line ) ;
                    break ;
                }
            }

        }
        m_tracks.Add( newtrack ) ;
        m_track_names.Add( m_track_names[i] ) ;
        *(tracks[i]) = m_tracks.GetCount()-1 ;
    }

}

void MidiFile::ReplicateTracks()
{
    if ( !m_has_easy )
    {
        if ( m_has_medium )
        {
            Replicate( 0, 1 ) ;
            m_has_easy = true ;
        }
        else if ( m_has_hard )
        {
            Replicate( 0, 2 ) ;
            m_has_easy = true ;
        }
        else if ( m_has_expert )
        {
            Replicate( 0, 3 ) ;
            m_has_easy = true ;
        }
    }
    if ( !m_has_medium )
    {
        if ( m_has_easy )
        {
            Replicate( 1, 0 ) ;
            m_has_medium = true ;
        }
        else if ( m_has_hard )
        {
            Replicate( 1, 2 ) ;
            m_has_medium = true ;
        }
        else if ( m_has_expert )
        {
            Replicate( 1, 3 ) ;
            m_has_medium = true ;
        }
    }
    if ( !m_has_hard )
    {
        if ( m_has_easy )
        {
            Replicate( 2, 0 ) ;
            m_has_hard = true ;
        }
        else if ( m_has_medium )
        {
            Replicate( 2, 1 ) ;
            m_has_medium = true ;
        }
        else if ( m_has_expert )
        {
            Replicate( 2, 3 ) ;
            m_has_medium = true ;
        }
    }
    if ( !m_has_expert )
    {
        if ( m_has_easy )
        {
            Replicate( 3, 0 ) ;
            m_has_expert = true ;
        }
        else if ( m_has_medium )
        {
            Replicate( 3, 1 ) ;
            m_has_expert = true ;
        }
        else if ( m_has_hard )
        {
            Replicate( 3, 2 ) ;
            m_has_expert = true ;
        }
    }
}


int MidiFile::writeGH2Format( wxString filename, int remove_orange, char has_coop )
{
    FILE *outfile ;
    wxString newheader ;
    wxString holder ;
    wxString line  ;
    wxString lastseq ;

    char szheader[20] ;

    if ( m_tracks.GetCount() == 0 )
    {
        return 1 ;
    }

    wxString tempfile = wxFileName::CreateTempFileName( _T("gh2makertmp") ) ;

    outfile = fopen( tempfile.char_str(), "wb" ) ;

    if ( outfile == NULL )
    {
        ::wxRemoveFile( tempfile ) ;
        return 1 ;
    }

    //newheader = m_header.BeforeFirst( ' ' ) ;
    holder = m_header.AfterFirst( ' ' ) ;

    //newheader.Append( _T( " " ) ) ;
    //newheader.Append( holder.BeforeFirst( ' ' ) ) ;
    //newheader.Append( _T( " " ) ) ;

    sprintf( szheader, "MFile 1 %d ", ( ( m_guitar_coop_track >= 0 ) || ( has_coop > 0 ) )? 9 : 8 ) ;

    newheader = wxString::FromAscii( szheader ) ;

    holder = holder.AfterFirst( ' ' ) ;
    newheader.Append( holder.AfterFirst( ' ' ) ) ;

    fprintf( outfile, "%s\n", (const char*)newheader.char_str() ) ;


    fprintf( outfile, "MTrk\n" ) ;
    fprintf( outfile, "0 Meta SeqName \"song\"\n" ) ;;

    if ( m_tempo_track >= 0 )
    {
		for ( unsigned int i = 0 ; i < m_tracks[0].GetCount() ; i++ )
		{
            fprintf( outfile, "%s\n", (const char*)m_tracks[0][i].char_str() ) ;;
		}
		if ( m_tracks[0].GetCount() == 0 )
            lastseq = _T("0") ;
        else
            lastseq = m_tracks[0][m_tracks[0].GetCount()-1].BeforeFirst( ' ') ;
    }
    else
    {
		fprintf( outfile, "0 TimeSig 4/4 24 8\n" ) ;
		fprintf( outfile, "0 Tempo 500000\n" ) ;
		for ( unsigned int i = 0 ; i < m_tempo_lines.GetCount() ; i++ )
		{
			fprintf( outfile, "%s\n", (const char*)m_tempo_lines[i].char_str() ) ;
		}
		if ( m_tempo_lines.GetCount() == 0 )
            lastseq = _T("0") ;
        else
            lastseq = m_tempo_lines[m_tempo_lines.GetCount()-1].BeforeFirst( ' ') ;
    }

    fprintf( outfile, "%s Meta TrkEnd\n", (const char*)lastseq.char_str() ) ;
    fprintf( outfile, "TrkEnd\n" ) ;


    if ( m_guitar_track == -1 )
        m_guitar_track = m_fallback_guitar_track ;

    ReplicateTracks() ;

    PrintNoteTrack( outfile, m_guitar_track, "PART GUITAR", remove_orange  ) ;


    if ( m_guitar_coop_track > -1 )
    {
        PrintNoteTrack( outfile, m_guitar_coop_track, "PART GUITAR COOP", remove_orange  ) ;
    }
    else if ( has_coop > 0 )
    {
        PrintNoteTrack( outfile, m_guitar_track, "PART GUITAR COOP", remove_orange  ) ;
    }

    if ( m_guitar_bass_track > -1 )
    {
        if ( m_tracks[m_guitar_bass_track].GetCount() < 100 )
        {
            m_guitar_bass_track = m_guitar_track ;
            m_track_names[m_guitar_bass_track] = (has_coop > 0) ? _T( "PART RHYTHM" ) : _T( "PART BASS" ) ;
        }
    }

    if ( m_guitar_bass_track > -1 )
    {
        if ( has_coop > 0 )
            PrintNoteTrack( outfile, m_guitar_bass_track, "PART RHYTHM", remove_orange  ) ;
        else
            PrintNoteTrack( outfile, m_guitar_bass_track, "PART BASS", remove_orange  ) ;
    }
    else
    {
        if ( has_coop > 0 )
            PrintNoteTrack( outfile, m_guitar_track, "PART RHYTHM", remove_orange  ) ;
        else
            PrintNoteTrack( outfile, m_guitar_track, "PART BASS", remove_orange  ) ;
    }



    if ( m_band_singer_track > -1 )
    {
		fprintf( outfile, "MTrk\n" ) ;
		fprintf( outfile, "0 Meta TrkName \"BAND SINGER\"\n" ) ;
        for  ( unsigned int i = 0 ; i < m_tracks[m_band_singer_track].GetCount() ; i++ )
        {
            fprintf( outfile, "%s\n", (const char*)m_tracks[m_band_singer_track][i].char_str() ) ;;
        }
		if ( m_tracks[m_band_singer_track].GetCount() == 0 )
            line = _T("0") ;
        else
            line = m_tracks[m_band_singer_track][m_tracks[m_band_singer_track].GetCount()-1].BeforeFirst(' ' ) ;
		fprintf( outfile, "%s Meta TrkEnd\n", (const char*)line.char_str()) ;
		fprintf( outfile, "TrkEnd\n" ) ;
    }
    else
    {
		fprintf( outfile, "MTrk\n" ) ;
		fprintf( outfile, "0 Meta TrkName \"BAND SINGER\"\n" ) ;
		fprintf( outfile, "5000 Meta Text \"[play]\"\n" ) ;
		fprintf( outfile, "5000 Meta TrkEnd\n" ) ;
		fprintf( outfile, "TrkEnd\n" ) ;
    }


    if ( m_band_drums_track > -1 )
    {
		fprintf( outfile, "MTrk\n" ) ;
		fprintf( outfile, "0 Meta TrkName \"BAND DRUMS\"\n" ) ;
        for  ( unsigned int i = 0 ; i < m_tracks[m_band_drums_track].GetCount() ; i++ )
        {
            fprintf( outfile, "%s\n", (const char*)m_tracks[m_band_drums_track][i].char_str() ) ;;
        }
		if ( m_tracks[m_band_drums_track].GetCount() == 0 )
            line = _T("0") ;
        else
            line = m_tracks[m_band_drums_track][m_tracks[m_band_drums_track].GetCount()-1].BeforeFirst(' ' ) ;
		fprintf( outfile, "%s Meta TrkEnd\n", (const char*)line.char_str()) ;
		fprintf( outfile, "TrkEnd\n" ) ;
    }
    else
    {
		fprintf( outfile, "MTrk\n" ) ;
		fprintf( outfile, "0 Meta TrkName \"BAND DRUMS\"\n" ) ;
		fprintf( outfile, "0 Meta Text \"[nobeat]\"\n" ) ;
		fprintf( outfile, "480 Meta Text \"[idle]\"\n" ) ;
		fprintf( outfile, "480 Meta TrkEnd\n" ) ;
		fprintf( outfile, "TrkEnd\n" ) ;
    }

    if ( m_band_bass_track > -1 )
    {
		fprintf( outfile, "MTrk\n" ) ;
		fprintf( outfile, "0 Meta TrkName \"BAND BASS\"\n" ) ;
        for  ( unsigned int i = 0 ; i < m_tracks[m_band_bass_track].GetCount() ; i++ )
        {
            fprintf( outfile, "%s\n", (const char*)m_tracks[m_band_bass_track][i].char_str() ) ;;
        }
		if ( m_tracks[m_band_bass_track].GetCount() == 0 )
            line = _T("0") ;
        else
            line = m_tracks[m_band_bass_track][m_tracks[m_band_bass_track].GetCount()-1].BeforeFirst(' ' ) ;
		fprintf( outfile, "%s Meta TrkEnd\n", (const char*)line.char_str()) ;
		fprintf( outfile, "TrkEnd\n" ) ;
    }
    else
    {
		fprintf( outfile, "MTrk\n" ) ;
		fprintf( outfile, "0 Meta TrkName \"BAND BASS\"\n" ) ;
		fprintf( outfile, "480 On ch=4 n=36 v=100\n" ) ;
		fprintf( outfile, "596 On ch=4 n=36 v=0\n" ) ;
		fprintf( outfile, "720 Meta Text \"[idle]\"\n" ) ;
		fprintf( outfile, "720 Meta TrkEnd\n" ) ;
		fprintf( outfile, "TrkEnd\n" ) ;
    }



    bool found_verse = false ;
    bool found_section = false ;
    bool found_end = false ;
    bool found_maxseq = false ;
    long currseq ;



    if ( m_events_track >= 0 )
    {
        for ( unsigned int i = 0 ; i < m_tracks[m_events_track].GetCount() ; i++ )
        {
            wxString line = (m_tracks[m_events_track])[i] ;

            if ( line.Contains( _T( "[verse]" ) ) )
                found_verse = true ;
            if ( line.Contains( _T( "[section" ) ) )
                found_section = true ;
            if ( line.Contains( _T( "[end]" ) ) )
            {
                found_end = true ;
                if ( ( line.BeforeFirst( ' ' ).ToLong( &currseq ) ) &&  ( currseq >= m_maxseq ) )
                    found_maxseq  = true ;
            }


        }
    }


    if ( ( m_events_track > -1 ) && found_verse && found_end && found_maxseq && found_section )
    {
		fprintf( outfile, "MTrk\n" ) ;
		fprintf( outfile, "0 Meta TrkName \"EVENTS\"\n" ) ;
        for  ( unsigned int i = 0 ; i < m_tracks[m_events_track].GetCount() ; i++ )
        {
            fprintf( outfile, "%s\n", (const char*)m_tracks[m_events_track][i].char_str() ) ;;
        }
		if ( m_tracks[m_events_track].GetCount() == 0 )
            line = _T("0") ;
        else
            line = m_tracks[m_events_track][m_tracks[m_events_track].GetCount()-1].BeforeFirst(' ' ) ;
		fprintf( outfile, "%s Meta TrkEnd\n", (const char*)line.char_str()) ;
		fprintf( outfile, "TrkEnd\n" ) ;
    }
    else
    {
		fprintf( outfile, "MTrk\n" ) ;
		fprintf( outfile, "0 Meta TrkName \"EVENTS\"\n" ) ;
		fprintf( outfile, "0 Meta Text \"[verse]\"\n" ) ;
		fprintf( outfile, "0 Meta Text \"[section intro]\"\n" ) ;
		fprintf( outfile, "%d Meta Text \"[end]\"\n", m_maxseq )  ;
		fprintf( outfile, "%d Meta TrkEnd\n", m_maxseq ) ;
		fprintf( outfile, "TrkEnd\n" ) ;
    }


    if ( m_triggers_track > -1 )
    {
		fprintf( outfile, "MTrk\n" ) ;
		fprintf( outfile, "0 Meta TrkName \"TRIGGERS\"\n" ) ;
        for  ( unsigned int i = 0 ; i < m_tracks[m_triggers_track].GetCount() ; i++ )
        {
            fprintf( outfile, "%s\n", (const char*)m_tracks[m_triggers_track][i].char_str() ) ;;
        }
		if ( m_tracks[m_triggers_track].GetCount() == 0 )
            line = _T("0") ;
        else
            line = m_tracks[m_triggers_track][m_tracks[m_triggers_track].GetCount()-1].BeforeFirst(' ' ) ;
		fprintf( outfile, "%s Meta TrkEnd\n", (const char*)line.char_str()) ;
		fprintf( outfile, "TrkEnd\n" ) ;
    }
    else
    {
		fprintf( outfile, "MTrk\n" ) ;
		fprintf( outfile, "0 Meta TrkName \"TRIGGERS\"\n" ) ;
		fprintf( outfile, "0 Meta TrkEnd\n" ) ;
		fprintf( outfile, "TrkEnd\n" ) ;
    }


    fclose( outfile ) ;

    switch ( midicomp_compile( tempfile.char_str(), filename.char_str()  ) )
    {
        case -1 :
        case -2 :
        {
            ::wxRemoveFile( tempfile ) ;
            return 1 ;
        }
    }


    ::wxRemoveFile( tempfile ) ;


    return 0 ;

}



bool MidiFile::isValid()
{
    bool found_verse = false ;
    bool found_section = false ;
    bool found_end = false ;
    bool found_maxseq = false ;
    long currseq ;

//all MIDIs need to be checked to ensure the data in them matches up to the SONGS.DTB file
    return false ;

/*
    currseq = 0 ;

    if ( ( m_guitar_track < 1 ) || ( m_guitar_bass_track < 1 ) || ( m_band_singer_track < 1 ) || ( m_band_drums_track < 1 ) ||
         ( m_band_bass_track < 1 ) || ( m_events_track < 1 ) || ( m_triggers_track < 1 ) || ( m_tempo_track < 0 ) ||
         ( !m_found_timesig ) || ( !m_found_tempo ) )
         return false ;


    if ( ( m_tracks.Item(m_band_singer_track).GetCount() < 5 ) || ( m_tracks[m_band_drums_track].GetCount() < 6 ) ||
         ( m_tracks[m_band_bass_track].GetCount() < 7 ) )
         return false ;

    for ( unsigned int i = 0 ; i < m_tracks[m_events_track].GetCount() ; i++ )
    {
        wxString line = (m_tracks[m_events_track])[i] ;

        if ( line.Contains( _T( "[verse]" ) ) )
            found_verse = true ;
        if ( line.Contains( _T( "[section" ) ) )
            found_section = true ;
        if ( line.Contains( _T( "[end]" ) ) )
        {
            found_end = true ;
            if ( ( line.BeforeFirst( ' ' ).ToLong( &currseq ) ) &&  ( currseq >= m_maxseq ) )
                found_maxseq  = true ;
        }


    }

    return found_verse && found_section && found_end && found_maxseq  ;
    */
}
