#include <wx/wxprec.h>

#ifdef __BORLANDC__
        #pragma hdrstop
#endif

#ifndef WX_PRECOMP
        #include <wx/wx.h>
#endif

#include "OggFile.h"

#ifdef __cplusplus
extern "C" {
#endif

void ogg_file_info(char *filename) ;
double ogg_file_info_get_time() ;
int ogg_file_info_get_channels() ;
int ogg_file_info_get_header() ;
int ogg_file_info_get_sample_rate() ;
char* ogg_file_info_get_artist() ;
char* ogg_file_info_get_title() ;

#ifdef __cplusplus
}
#endif




OggFile::OggFile ()
{
    m_opened = false ;
    m_total_pcm_size = 0 ;
    m_total_decoded = 0 ;
    m_num_blank = m_num_skip = 0 ;
}

void OggFile::SetDelay( int delay )
{
    float fdelay = delay ;

    if ( m_num_channels == 0 )
        return ;

    if ( delay < 0 )
    {
        m_num_blank = (int)((fdelay*((float)m_sample_rate)*-2.0f*((float)m_num_channels))/1000.0f) ;

        if ( ( m_num_blank % ( 2*m_num_channels) ) > 0 )
            m_num_blank += (2*m_num_channels) - ( m_num_blank % ( 2*m_num_channels) ) ;
    }
    else
    {
        m_num_skip = (int)((fdelay*((float)m_sample_rate)*2.0f*((float)m_num_channels))/1000.0f) ;

        if ( ( m_num_skip % ( 2*m_num_channels) ) > 0 )
            m_num_skip += (2*m_num_channels) - ( m_num_skip % ( 2*m_num_channels) ) ;
    }
}

void OggFile::SetFile(wxString filename)
{
    ogg_file_info( filename.char_str() ) ;
    m_num_seconds =  ogg_file_info_get_time() ;
    m_num_channels = ogg_file_info_get_channels() ;
    m_sample_rate = ogg_file_info_get_sample_rate() ;
    m_artist = wxString::FromAscii( ogg_file_info_get_artist() ) ;
    m_title = wxString::FromAscii( ogg_file_info_get_title() ) ;
    m_has_header = ogg_file_info_get_header() ;
    m_filename = filename ;

}

int OggFile::GetDecodedOggBytes( unsigned char *buf, unsigned int count )
{
    int bs = 0;
    int ret;
    int numread ;
    int numleft ;
    char *tmpbuf ;

    if ( !m_opened )
    {
        if ( OpenOgg() )
            return 0 ;
    }

    if ( m_num_skip )
    {
        numleft = m_num_skip ;
        numread = 0 ;

        tmpbuf = (char*)malloc(m_num_skip) ;

        while((ret = ov_read(&m_vf, (char*)(tmpbuf+numread), numleft, 0, 2, 1, &bs)) != 0) {
            if(bs != 0) {
                vorbis_info *vi = ov_info(&m_vf, -1);
                if(m_num_channels!= vi->channels || m_sample_rate != vi->rate) {
                    //fprintf(stderr, "Logical bitstreams with changing parameters are not supported\n");
                    return 0 ;
                }
            }

            if(ret < 0 ) {
                continue;
            }

            numread += ret ;
            numleft -= ret ;

            if ( numleft <= 0 )
                break ;
        }

        free(tmpbuf) ;
        m_num_skip = 0 ;
    }

    if ( m_num_blank )
    {
        if ( count >= (unsigned int)m_num_blank )
        {
            memset( buf, 0, m_num_blank ) ;
            numleft = count-m_num_blank ;
            numread = m_num_blank ;
            m_num_blank = 0 ;

        }
        else
        {
            memset( buf, 0, count) ;
            m_num_blank -= count ;
            return count ;
        }

    }
    else
    {
        numleft = count ;
        numread = 0 ;
    }



    while((ret = ov_read(&m_vf, (char*)(buf+numread), numleft, 0, 2, 1, &bs)) != 0) {
        if(bs != 0) {
            vorbis_info *vi = ov_info(&m_vf, -1);
            if(m_num_channels!= vi->channels || m_sample_rate != vi->rate) {
                //fprintf(stderr, "Logical bitstreams with changing parameters are not supported\n");
                return 0 ;
            }
        }

        if(ret < 0 ) {
            continue;
        }

        numread += ret ;
        numleft -= ret ;

        if ( numleft <= 0 )
            break ;
    }

    m_total_decoded += numread ;

    return numread;
}

OggFile::~OggFile ()
{
    if ( m_opened )
        ov_clear( &m_vf ) ;
}

int OggFile::getNumChannels()
{
    return m_num_channels;
}

bool OggFile::isValid()
{
    return ( m_num_channels == 6 ) ;
}



int OggFile::OpenOgg(  )
{
    FILE *infile ;

    infile = fopen(m_filename.char_str(), "rb");

    if(!infile) {
        return 1;
    }

    if(ov_open(infile, &m_vf, NULL, 0) < 0) {
        //fprintf(stderr, "ERROR: Failed to open input as vorbis\n");
        fclose(infile);
        return 1;
    }

    m_opened = true ;

    if(ov_seekable(&m_vf)) {
        int link;
        int chainsallowed = 0;
        for(link = 0; link < ov_streams(&m_vf); link++) {
            if(ov_info(&m_vf, link)->channels == m_num_channels &&
                    ov_info(&m_vf, link)->rate == m_sample_rate)
            {
                chainsallowed = 1;
            }
        }

        if(chainsallowed)
            m_total_pcm_size = ov_pcm_total(&m_vf, -1);
        else
            m_total_pcm_size = ov_pcm_total(&m_vf, 0);

        m_total_pcm_size *= ( m_num_channels * 2 ) ;
    }


    ov_pcm_seek( &m_vf, 0 ) ;

    return 0;
}

