#include <wx/wxprec.h>

#ifdef __BORLANDC__
        #pragma hdrstop
#endif

#ifndef WX_PRECOMP
        #include <wx/wx.h>
#endif

#include "OggFile.h"
#include "OggMerger.h"
#include "ProgressDialog.h"

extern int g_current_gauge_value ;
extern int g_overall_gauge_value ;
extern wxSemaphore g_semProgress;

#ifdef __cplusplus
extern "C" {
#endif

int ogg_encode( int sample_rate, int quality, char *outfile, char *artist, char *title ) ;
OggMerger *g_merger ;

int g_tot_pcm = 0 ;
int g_enc_pcm = 0 ;

long read_pcm_bytes( unsigned char *buf, int element_size, int count, FILE *infile)
{
    int numchannels ;
    int numsamples ;
    int maxread = 0 ;
    int counts[3] ;
    unsigned int stride_src ;
    OggFile *ogg ;
    unsigned int firstidx ;
    unsigned short *src ;
    unsigned short  *dest ;
    unsigned char *streams[3] ;


    if ( g_merger->m_notify_dlg->m_done )
    {
        return -1 ;
    }

    streams[0] = NULL ;
    streams[1] = NULL ;
    streams[2] = NULL ;
    firstidx = 0 ;


    numsamples = ( element_size * count ) / 12 ;

    g_enc_pcm += (count*element_size) ;

    for ( int i = 0 ; i < 3 ; i++ )
    {
        if ( ( numchannels = ( g_merger->m_ogg_files[i].m_num_channels ) ) )
        {
            firstidx = i ;
            streams[i] = (unsigned char*)( malloc( numsamples * numchannels * 2 ) ) ;
            counts[i] = g_merger->m_ogg_files[i].GetDecodedOggBytes( streams[i], ( numsamples * numchannels * 2  ) );
            if ( ( counts[i] / ( numchannels*2) ) > maxread )
                maxread = ( counts[i] / ( numchannels*2) );
            g_tot_pcm += counts[i] ;
            memset( streams[i] + counts[i], 0, (numsamples*numchannels*2 )-counts[i] ) ;

        }
    }

    if ( maxread == 0 )
    {
        for ( int i = 0 ; i < 3 ; i++ )
        {
            if ( streams[i] )
                free( streams[i] ) ;
        }
        return 0 ;
    }


    for ( int i = 0 ; i < 6 ; i++ )
    {
        src  = (unsigned short*)(streams[g_merger->m_file_index[i]] + ( g_merger->m_file_channel[i] * 2 )) ;
        dest = (unsigned short*)(buf + ( i*2 )) ;

        ogg = &(g_merger->m_ogg_files[g_merger->m_file_index[i]]) ;
        stride_src = ogg->m_num_channels  ;

        for ( int j = 0 ; j < numsamples ; j++ )
        {
            *dest = *src ;
            src += stride_src ; ;
            dest += 6 ;
        }
    }


    for ( int i = 0 ; i < 3 ; i++ )
    {
        if ( streams[i] )
            free( streams[i] ) ;
    }


    {
        int pct ;
        float fpct ;

        //wxMutexGuiEnter();


        wxCommandEvent event( wxEVT_CURRENT_PROGRESS_EVENT, g_merger->m_notify_id  );
        fpct = (float)( g_merger->m_ogg_files[firstidx].m_total_decoded ) / (float)(g_merger->m_ogg_files[firstidx].m_total_pcm_size) ;
        pct = (int)(fpct*100.0f) ;
        event.SetInt( pct ) ;
        event.SetString( wxString( _T("Creating ") ).Append( g_merger->m_output_filename ) ) ;
        wxPostEvent( g_merger->m_notify_dlg, event );


        if ( g_merger->m_standalone )
        {
            wxCommandEvent event( wxEVT_OVERALL_PROGRESS_EVENT, g_merger->m_notify_id  );
            event.SetInt( pct ) ;
            event.SetString( wxString( _T("Overall Progress") ) ) ;
            wxPostEvent( g_merger->m_notify_dlg, event );
        }
    /*
        wxCommandEvent event( wxEVT_MY_EVENT, g_merger->m_notify_id );
        event.SetInt( ( g_merger->m_ogg_files[firstidx].m_total_decoded * 100 ) / g_merger->m_ogg_files[firstidx].m_total_pcm_size ) ;
        if ( ( ( g_merger->m_ogg_files[firstidx].m_total_decoded * 100 ) / g_merger->m_ogg_files[firstidx].m_total_pcm_size ) > 100 )
           streams[0] = NULL ;

        event.SetEventObject( g_merger->m_notify_dlg );
        g_merger->m_notify_dlg->GetEventHandler()->ProcessEvent( event );
    */

        //wxMutexGuiLeave();
    }

    return maxread*12 ;
}
#if 0

long read_pcm_bytes_rockband( unsigned char *buf, int element_size, int count, FILE *infile)
{
    int numchannels ;
    int numsamples ;
    int maxread = 0 ;
    int counts[3] ;
    float sample ;
    unsigned int stride_src ;
    OggFile *ogg ;
    unsigned int firstidx ;
    unsigned short *src ;
    unsigned short  *dest ;
    unsigned char *streams[3] ;


    if ( g_merger->m_notify_dlg->m_done )
    {
        return -1 ;
    }

    streams[0] = NULL ;
    streams[1] = NULL ;
    streams[2] = NULL ;
    firstidx = 0 ;


    numsamples = ( element_size * count ) / 12 ;

    g_enc_pcm += (count*element_size) ;

    for ( int i = 0 ; i < 3 ; i++ )
    {
        if ( ( numchannels = ( g_merger->m_ogg_files[i].m_num_channels ) ) )
        {
            firstidx = i ;
            if ( numchannels < 12 )
                return 0 ;
            streams[i] = (unsigned char*)( malloc( numsamples * numchannels * 2 ) ) ;
            counts[i] = g_merger->m_ogg_files[i].GetDecodedOggBytes( streams[i], ( numsamples * numchannels * 2  ) );
            if ( ( counts[i] / ( numchannels*2) ) > maxread )
                maxread = ( counts[i] / ( numchannels*2) );
            g_tot_pcm += counts[i] ;
            memset( streams[i] + counts[i], 0, (numsamples*numchannels*2 )-counts[i] ) ;

        }
    }

    if ( maxread == 0 )
    {
        for ( int i = 0 ; i < 3 ; i++ )
        {
            if ( streams[i] )
                free( streams[i] ) ;
        }
        return 0 ;
    }


    for ( int i = 0 ; i < 6 ; i++ )
    {
        src  = (unsigned short*)(streams[0] ) ;
        dest = (unsigned short*)(buf + ( i*2 )) ;

        ogg = &(g_merger->m_ogg_files[g_merger->m_file_index[i]]) ;
        stride_src = ogg->m_num_channels  ;

        for ( int j = 0 ; j < numsamples ; j++ )
        {
            switch ( i )
            {
/*
0 - drums, mono -> song L, song R
1 - drums2, L -> song L
2 - drums2, R -> song R
3 - drums3, L -> song L
4 - drums3, R -> song R
5 - bass, mono -> bass L, bass R
6 - guitar L -> guitar L
7 - guitar R -> guitar R
8 - vocals L -> song L
9 - vocals R -> song R
10 - extra vocals L ->song L
11 - extra vocals R -> song R
*/
                case 0 : //Song L
                {
                    //0 + 1 + 3 + 8 + 10
                    sample = (*src + *(src+1) + *(src+3) + *(src+8) + *(src+10))/5.0f
                    break ;
                }
                case 1 : //Song R
                {
                    //0 + 2 + 4 + 9 + 11
                    sample = (*src + *(src+2) + *(src+4) + *(src+9) + *(src+11))/5.0f
                    break ;
                }
                case 2 : //Guitar L
                {
                    //6
                    sample = *(src+6) ;
                    break ;
                }
                case 3 : //Guitar R
                {
                    //7
                    sample = *(src+7) ;
                    break ;
                }
                case 4 : //Bass L
                case 5 : //Bass R
                {
                    //5
                    sample = *(src+5) ;
                    break ;
                }
            }

            *dest = sample ;
            src += stride_src ; ;
            dest += 6 ;
        }
    }


    for ( int i = 0 ; i < 3 ; i++ )
    {
        if ( streams[i] )
            free( streams[i] ) ;
    }


    {
        int pct ;
        float fpct ;

        //wxMutexGuiEnter();


        wxCommandEvent event( wxEVT_CURRENT_PROGRESS_EVENT, g_merger->m_notify_id  );
        fpct = (float)( g_merger->m_ogg_files[firstidx].m_total_decoded ) / (float)(g_merger->m_ogg_files[firstidx].m_total_pcm_size) ;
        pct = (int)(fpct*100.0f) ;
        event.SetInt( pct ) ;
        event.SetString( wxString( _T("Creating ") ).Append( g_merger->m_output_filename ) ) ;
        wxPostEvent( g_merger->m_notify_dlg, event );


        if ( g_merger->m_standalone )
        {
            wxCommandEvent event( wxEVT_OVERALL_PROGRESS_EVENT, g_merger->m_notify_id  );
            event.SetInt( pct ) ;
            event.SetString( wxString( _T("Overall Progress") ) ) ;
            wxPostEvent( g_merger->m_notify_dlg, event );
        }
    /*
        wxCommandEvent event( wxEVT_MY_EVENT, g_merger->m_notify_id );
        event.SetInt( ( g_merger->m_ogg_files[firstidx].m_total_decoded * 100 ) / g_merger->m_ogg_files[firstidx].m_total_pcm_size ) ;
        if ( ( ( g_merger->m_ogg_files[firstidx].m_total_decoded * 100 ) / g_merger->m_ogg_files[firstidx].m_total_pcm_size ) > 100 )
           streams[0] = NULL ;

        event.SetEventObject( g_merger->m_notify_dlg );
        g_merger->m_notify_dlg->GetEventHandler()->ProcessEvent( event );
    */

        //wxMutexGuiLeave();
    }

    return maxread*12 ;
}
#endif
#ifdef __cplusplus
}
#endif



void OggMerger::OnExit()
{
    g_semProgress.Post() ;
}


wxThread::ExitCode OggMerger::Entry()
{

    g_semProgress.Wait() ;
    /*
    while ( ! m_notify_dlg->m_done )
    {

        g_current_gauge_value = rand()%100 ;

        wxCommandEvent event( wxEVT_MY_EVENT, m_notify_id  );
        event.SetInt( rand()%100 );
        wxPostEvent( m_notify_dlg, event );


        Sleep(100) ;
    }
*/
    if ( ogg_encode( m_samplerate, m_quality /*quality*/, m_output_filename.char_str(), m_artist.char_str(), m_title.char_str()) )
    {
        wxCommandEvent event( wxEVT_FINISHED_EVENT, g_merger->m_notify_id  );
        event.SetInt( 1 ) ;
        wxPostEvent( g_merger->m_notify_dlg, event );
        m_success = false ;
    }
    else
    {
        wxCommandEvent event( wxEVT_FINISHED_EVENT, g_merger->m_notify_id  );
        event.SetInt( 0 ) ;
        wxPostEvent( g_merger->m_notify_dlg, event );
        m_success = true ;
    }


    m_done = true ;


    //g_semProgress.Post() ;

    return 0 ;
}


OggMerger::OggMerger()
{
    g_merger = this ;
    m_samplerate = 0 ;
    wxThread::Create() ;
    m_done = false ;
    m_standalone = true ;
    m_success = false ;
    m_quality = 3 ;
    m_delay = 0 ;
}

int OggMerger::SetData( wxString oggfile1, wxString oggfile2, wxString oggfile3,
                        int channel_1_file, int channel_1_channel,
                        int channel_2_file, int channel_2_channel,
                        int channel_3_file, int channel_3_channel,
                        int channel_4_file, int channel_4_channel,
                        int channel_5_file, int channel_5_channel,
                        int channel_6_file, int channel_6_channel, wxString outfile, wxString artist, wxString title,
                        int delay, int quality, ProgressDialog *dlg, int id, bool standalone, bool isrockband  )
{

    m_standalone = standalone ;

    m_quality = quality ;

    m_output_filename = outfile ;
    m_artist = artist ;
    m_title = title ;
    m_delay = delay  ;

    m_ogg_files[0].SetFile( oggfile1 ) ;
    m_ogg_files[1].SetFile( oggfile2 ) ;
    m_ogg_files[2].SetFile( oggfile3 ) ;

    m_ogg_files[0].SetDelay( delay );
    m_ogg_files[1].SetDelay( delay );
    m_ogg_files[2].SetDelay( delay );

    m_file_index[0] = channel_1_file ;
    m_file_index[1] = channel_2_file ;
    m_file_index[2] = channel_3_file ;
    m_file_index[3] = channel_4_file ;
    m_file_index[4] = channel_5_file ;
    m_file_index[5] = channel_6_file ;

    m_file_channel[0] = channel_1_channel ;
    m_file_channel[1] = channel_2_channel ;
    m_file_channel[2] = channel_3_channel ;
    m_file_channel[3] = channel_4_channel ;
    m_file_channel[4] = channel_5_channel ;
    m_file_channel[5] = channel_6_channel ;

    m_notify_dlg = dlg ;
    m_notify_id = id ;


    if
    (
        ( ( oggfile1.Length() > 0 ) && ( m_ogg_files[0].getNumChannels() == 0 ) ) ||
        ( ( oggfile2.Length() > 0 ) && ( m_ogg_files[1].getNumChannels() == 0 ) ) ||
        ( ( oggfile3.Length() > 0 ) && ( m_ogg_files[2].getNumChannels() == 0 ) )
    )
    {
        return 1 ;
    }

    for ( int i = 0 ; i < 6 ; i++ )
    {
        if ( m_ogg_files[m_file_index[i]].getNumChannels() < m_file_channel[i] )
            return 2 ;
    }


    if ( m_ogg_files[0].m_sample_rate > m_samplerate )
        m_samplerate = m_ogg_files[0].m_sample_rate ;
    if ( m_ogg_files[1].m_sample_rate > m_samplerate )
        m_samplerate = m_ogg_files[1].m_sample_rate ;
    if ( m_ogg_files[2].m_sample_rate > m_samplerate )
        m_samplerate = m_ogg_files[2].m_sample_rate ;

    if ( ( m_ogg_files[0].m_sample_rate != m_samplerate ) ||
         ( m_ogg_files[1].m_sample_rate != m_samplerate ) ||
         ( m_ogg_files[2].m_sample_rate != m_samplerate ) )
        return 3 ;


    return 0 ;

}

int OggMerger::Merge()
{
    m_success = ( ogg_encode( m_samplerate, m_quality /*quality*/, m_output_filename.char_str(), m_artist.char_str(), m_title.char_str() ) == 0 );
    return m_success ;
}

OggMerger::~OggMerger()
{

}
