#include <wx/wxprec.h>

#ifdef __BORLANDC__
	#pragma hdrstop
#endif

#ifndef WX_PRECOMP
	#include <wx/wx.h>
#endif

#include <wx/textfile.h>

#include "inifile.h"
#include <string.h>
#include <math.h>


#ifndef min
#define min(a,b) (((a) < (b)) ? (a) : (b))
#endif

#ifndef max
#define max(a,b) (((a) > (b)) ? (a) : (b))
#endif

IniFile::IniFile (wxString FileName)
{
    FILE *inifile ;
    char *buf ;
    int filesize ;

    m_iniHolder = NULL ;
    m_iniSize = 0 ;

	IniFileName = FileName;

	if ( ( inifile = fopen( FileName.char_str(), "rb" ) ) )
	{
		fseek( inifile, 0, SEEK_END ) ;
		filesize = ftell( inifile ) ;
		fseek( inifile, 0, SEEK_SET ) ;

		buf = (char*)malloc( filesize+2 ) ;
		memset(buf, 0, filesize+2 ) ;

		fread( buf, sizeof(char), filesize, inifile ) ;
		fclose( inifile ) ;

		fillIniStruct( buf );

		free(buf) ;
	}
}


IniFile::~IniFile()
{
    fillIniStruct( NULL );

}


wxString IniFile::GetIniString( char *section, char *setting, char *default_val )
{
	char bsection[100] ;
	char bsetting[100] ;
	int sectionnum = 0 ;

	int sectionStart = 0 ;

	if ( m_iniSize == 0 )
	{
		return wxString::FromAscii( default_val )  ;
	}

    strcpy( bsection, wxString::FromAscii( section ).MakeUpper().char_str() ) ;
    strcpy( bsetting, wxString::FromAscii( setting ).MakeUpper().char_str() ) ;


	sectionnum = m_iniHolder[0].sectionnum ;

	while ( strcmp( bsection, m_iniHolder[sectionStart].section ) )
	{
		sectionnum = m_iniHolder[sectionStart].sectionnum ;

		while ( ( sectionStart < m_iniSize ) && ( sectionnum == m_iniHolder[sectionStart].sectionnum ) )
		{
			sectionStart++ ;
		}


		if ( sectionStart >= m_iniSize )
		{
            return wxString::FromAscii( default_val ) ;
		}
	}

	sectionnum = m_iniHolder[sectionStart].sectionnum ;

	if ( sectionStart >= m_iniSize )
	{
		return wxString::FromAscii( default_val ) ;
	}

	while ( strcmp( bsetting, m_iniHolder[sectionStart].setting ) )
	{
		sectionStart++ ;

		if ( ( sectionStart >= m_iniSize ) || ( m_iniHolder[sectionStart].sectionnum != sectionnum ) )
		{
            return wxString::FromAscii( default_val ) ;
		}
	}

    return wxString::FromAscii( m_iniHolder[sectionStart].value ) ;

}

int IniFile::GetIniInt( char *section, char *setting, int default_val )
{
    wxString strValue ;
	int val ;


	strValue = GetIniString( section, setting, "X" ) ;

	if ( strValue.GetChar( 0 ) == 'X' )
		return default_val ;

    val = atoi( strValue.char_str() ) ;

	return val ;

}

int IniFile::getLine( char *pos, char *line, unsigned int linesize )
{
	unsigned int linepos ;
	int advance ;

	linepos = 0 ;
	advance = 0 ;


	memset( line, 0, linesize ) ;

	if ( !pos )
		return advance ;

	while ( *pos && ( isspace( *pos ) ) )
	{
		pos++ ;
		advance++ ;
	}

	if ( ! *pos )
		return advance;

	while ( *pos && ( *pos != '\r' ) && ( *pos != '\n' )  && ( linepos < ( linesize-2) ) )
	{
		line[linepos++] = *pos ;
		pos++ ;
		advance++ ;
	}

	line[linepos] = 0 ;

	while ( linepos && ( ( line[linepos-1] == ' ' ) || ( line[linepos-1]== '\t' ) ) )
	{
		line[linepos-1] = 0 ;
		linepos-- ;
	}

	return advance ;
}

void IniFile::fillIniStruct( char *buf )
{
	char section[100] ;
	char setting[100] ;
	char value[500] ;
	char line[505] ;
	int section_pos ;
	int setting_pos ;
	int advance ;
	char *ptr ;
	int section_num ;
	char *secpos_b ;
	char *secpos_e ;
	int  linepos ;

	if ( m_iniHolder != NULL )
	{
		for ( int i = 0 ; i < m_iniSize ; i++ )
		{
			free( m_iniHolder[i].section ) ;
			free( m_iniHolder[i].setting ) ;
			free( m_iniHolder[i].value ) ;
		}

		free( m_iniHolder ) ;
		m_iniHolder = NULL ;
		m_iniSize = 0 ;
	}


	if ( buf == NULL )
		return ;

	section_num = 0 ;


	ptr = buf ;

	while ( ( advance = getLine( ptr, line, 500 ) )  )
	{
		ptr += advance ;

		if ( line[0] == '#' )
			continue ;

		if ( line[0] == '[' )
		{

			secpos_b = line ;

			section_pos = 0 ;
			setting_pos = 0 ;

			secpos_e = secpos_b+1 ;

			while ( *secpos_e && ( *secpos_e != ']' ) && ( section_pos < 99 ) )
			{
				if ( ! isspace( *secpos_e ) )
				{
					section[section_pos++] = *secpos_e ;
				}

				secpos_e++ ;
			}

			if ( *secpos_e != ']' )
			{
				section[0] = 0 ;
				section_pos = 0 ;
				continue ;
			}

			section[section_pos] = 0 ;
            strcpy( section, wxString::FromAscii( section ).MakeUpper().char_str() ) ;
			section_num++ ;

			continue ;
		}

		if ( ( secpos_e = strchr( line, '=' ) ) )
		{
			strncpy( setting, line, min( 99, secpos_e-line) ) ;
			setting[ min( 99, secpos_e - line) ] = 0 ;

			linepos = strlen( setting )  ;
			while ( linepos && ( ( setting[linepos-1] == ' ' ) || ( setting[linepos-1]== '\t' ) ) )
			{
				setting[linepos-1] = 0 ;
				linepos-- ;
			}

			strncpy( value, secpos_e+1, min( 499, strlen( secpos_e +1 ) ) ) ;
			value[ min( 499, strlen( secpos_e+1 ) ) ] = 0 ;

			linepos = strlen( value )  ;
			while ( linepos && ( ( value[linepos-1] == ' ' ) || ( value[linepos-1]== '\t' ) ) )
			{
				value[linepos-1] = 0 ;
				linepos-- ;
			}

            strcpy( setting, wxString::FromAscii( setting ).MakeUpper().char_str() ) ;

			m_iniHolder = (INISTRUCT*)realloc( m_iniHolder, sizeof(INISTRUCT) * ( m_iniSize  + 1 ) ) ;

			m_iniHolder[m_iniSize].section = (char*)malloc( strlen(section)+1 ) ;
			m_iniHolder[m_iniSize].setting = (char*)malloc( strlen(setting)+1 ) ;
			m_iniHolder[m_iniSize].value = (char*)malloc( strlen(value)+1 ) ;
			m_iniHolder[m_iniSize].sectionnum = section_num ;

			strcpy(m_iniHolder[m_iniSize].section , section ) ;
			strcpy(m_iniHolder[m_iniSize].setting, setting ) ;
			strcpy(m_iniHolder[m_iniSize].value , value ) ;

			m_iniSize++ ;


		}

	}


}


