/* OggDec
 *
 * This program is distributed under the GNU General Public License, version 2.
 * A copy of this license is included with this source.
 *
 * Copyright 2002, Michael Smith <msmith@xiph.org>
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>
#include <stdio.h>
#ifndef WIN32
#include <getopt.h>
#endif
#include <errno.h>
#include <string.h>

#if defined(_WIN32) || defined(__EMX__) || defined(__WATCOMC__)
#include <fcntl.h>
#include <io.h>
#endif

#include <vorbis/vorbisfile.h>

#include "i18n.h"


//static int quiet = 0;
static int bits = 16;
//static int endian = 0;
static int raw = 0;
//static int sign = 1;
unsigned char headbuf[44]; /* The whole buffer */
char *outfilename = NULL;

/*
static int decode_file(FILE *in, FILE *out, char *infile, char *outfile)
{
    OggVorbis_File vf;
    int bs = 0;
    char buf[8192] ; //, outbuf[8192];
//    char *p_outbuf;
    int buflen = 8192;
    unsigned int written = 0;
    int ret;
    ogg_int64_t length = 0;
    ogg_int64_t done = 0;
    int size = 0;
    int seekable = 0;
    int percent = 0;
    int channels;
    int samplerate;

    if(ov_open(in, &vf, NULL, 0) < 0) {
        fprintf(stderr, "ERROR: Failed to open input as vorbis\n");
        fclose(in);
        return 1;
    }

    channels = ov_info(&vf,0)->channels;
    samplerate = ov_info(&vf,0)->rate;

    if(ov_seekable(&vf)) {
        int link;
        int chainsallowed = 0;
        for(link = 0; link < ov_streams(&vf); link++) {
            if(ov_info(&vf, link)->channels == channels &&
                    ov_info(&vf, link)->rate == samplerate)
            {
                chainsallowed = 1;
            }
        }

        seekable = 1;
        if(chainsallowed)
            length = ov_pcm_total(&vf, -1);
        else
            length = ov_pcm_total(&vf, 0);
        size = bits/8 * channels;
    }


    while((ret = ov_read(&vf, buf, buflen, 0, 2, 1, &bs)) != 0) {
        if(bs != 0) {
            vorbis_info *vi = ov_info(&vf, -1);
            if(channels != vi->channels || samplerate != vi->rate) {
                fprintf(stderr, "Logical bitstreams with changing parameters are not supported\n");
                break;
            }
        }

        if(ret < 0 ) {
            continue;
        }

        written += ret;
        if(!quiet && seekable) {
            done += ret/size;
            if((double)done/(double)length * 200. > (double)percent) {
                percent = (double)done/(double)length *200;
                fprintf(stderr, "\r\t[%5.1f%%]", (double)percent/2.);
            }
        }
    }

    if(seekable && !quiet)
        fprintf(stderr, "\n");


    ov_clear(&vf);

    return 0;
}
*/

int ogg_read_bytes(OggVorbis_File *vf, unsigned char *buf, unsigned int buflen, int channels, int samplerate )
{
    int bs = 0;
    unsigned int written = 0;
    int ret;
//    ogg_int64_t done = 0;
//    int size = 0;
//    int seekable = 0;
//    int percent = 0;



    while((ret = ov_read(vf, (char*)buf, buflen, 0, 2, 1, &bs)) != 0) {
        if(bs != 0) {
            vorbis_info *vi = ov_info(vf, -1);
            if(channels != vi->channels || samplerate != vi->rate) {
                //fprintf(stderr, "Logical bitstreams with changing parameters are not supported\n");
                return 0 ;
            }
        }

        if(ret < 0 ) {
            continue;
        }

        written += ret;
    }



    return 0;
}

int ogg_finish_decode_file( OggVorbis_File *vf )
{
    if ( vf->datasource )
        fclose( vf->datasource ) ;

    ov_clear(vf);

    return 0 ;
}

int ogg_start_decode_file( OggVorbis_File *vf, char *infilename, unsigned int *totalsize )
{
    int channels;
    int samplerate;
    FILE *infile; //, *outfile;

    bits = 16;
    raw = 1;



    infile = fopen(infilename, "rb");

    if(!infile) {
        return 1;
    }

    if(ov_open(infile, vf, NULL, 0) < 0) {
        //fprintf(stderr, "ERROR: Failed to open input as vorbis\n");
        fclose(infile);
        return 1;
    }

    channels = ov_info(vf,0)->channels;
    samplerate = ov_info(vf,0)->rate;

    if(ov_seekable(vf)) {
        int link;
        int chainsallowed = 0;
        for(link = 0; link < ov_streams(vf); link++) {
            if(ov_info(vf, link)->channels == channels &&
                    ov_info(vf, link)->rate == samplerate)
            {
                chainsallowed = 1;
            }
        }

        if(chainsallowed)
            *totalsize = ov_pcm_total(vf, -1);
        else
            *totalsize = ov_pcm_total(vf, 0);
    }


    return 0;
}

