#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define StopProgram() getc(stdin)

#define FILENAME_BUFFER 1000
#define MAX_BLOCK_SIZE 0x10000

/*
Some useful URLs:

http://www.irisa.fr/texmex/people/dufouil/ffmpegdoxy/mpeg_8c-source.html
*/

/* MPEG codes */
#define CODE_START 0xBA010000UL
#define CODE_HEADER_START 0xBB010000UL
#define CODE_VIDEO 0xE0010000UL
#define CODE_PADDING 0xBE010000UL
#define CODE_PRIVATE_STREAM_1 0xBD010000UL
#define CODE_END 0xB9010000UL

/* Swap endian of 16-bit short */
inline void EndianSwapShort(unsigned short *x)
{
    *x = (*x >> 8) | (*x << 8);
}

/* Swap endian of 32-bit integer */
inline void EndianSwapInt(unsigned int *x)
{
    *x = (*x >> 24) | ((*x << 8) & 0x00FF0000) | ((*x >> 8) & 0x0000FF00) | (*x << 24);
}

/* Read for big endian */
size_t freadBe(void *ptr, size_t size, size_t count, FILE *stream)
{
    size_t returnCode, i;

    /* Read */
    returnCode = fread(ptr, size, count, stream);

    /* Swap endian */
    if (size == 4)
    {
        unsigned int *ptrInt;

        ptrInt = (unsigned int *) ptr;

        for (i = 0; i < count; i++)
        {
            EndianSwapInt(&ptrInt[i]);
        }
    }
    else if (size == 2)
    {
        unsigned short *ptrShort;

        ptrShort = (unsigned short *) ptr;

        for (i = 0; i < count; i++)
        {
            EndianSwapShort(&ptrShort[i]);
        }
    }
    else if (size != 1)
    {
        printf("WARNING: Endian swap not supported (Size = %i, count = %i)\n", size, count);
        StopProgram();
        exit(-1);
    }

    return returnCode;
}

int main(void)
{
    /* Input/Output files */
    char mode;

    FILE *pssFile, *vgsFile;
    char pssFileName[FILENAME_BUFFER], vgsFileName[FILENAME_BUFFER];
    long int vgsFileSize, vgsCopiedSize = 0;

    /* Block info */
    unsigned int blockType;
    unsigned short blockSize;

    /* Title */
    printf("PSSInject 0.1\n-------------\n(Thanks to xorloser for his PSSTool!)\n\n");

    /* Request mode */
    do
    {
        printf("Mode (e = extract VGS, i = inject VGS): ");
        mode = getc(stdin);
        fflush(stdin);
    } while (mode != 'e' && mode != 'i');

    /* Request PSS filename */
    printf("PSS filename: ");
    gets(pssFileName);

    /* Open PSS */
    pssFile = fopen(pssFileName, "rb+");
    if (pssFile == NULL)
    {
        printf("Can't open the PSS file \"%s\"!\n", pssFileName);
        StopProgram();
        exit(-1);
    }

    /* Request VGS filename */
    printf("VGS filename: ");
    gets(vgsFileName);

    /* Open VGS */
    if (mode == 'e') // Extract = write access
        vgsFile = fopen(vgsFileName, "wb");
    else if (mode == 'i') // Inject = read access
        vgsFile = fopen(vgsFileName, "rb");

    if (vgsFile == NULL)
    {
        printf("Can't open the VGS file \"%s\"!\n", vgsFileName);
        StopProgram();
        exit(-1);
    }

    /* Get VGS filesize */
    fseek(vgsFile, 0, SEEK_END);
    vgsFileSize = ftell(vgsFile);
    fseek(vgsFile, 0, SEEK_SET);

    /* Read headers */
    while (1 == 1)
    {
        /* Read type */
        fread(&blockType, 4, 1, pssFile);

        /* Check if it's EOF */
        if (feof(pssFile) != 0)
        {
            printf("WARNING: EOF reached without end code!\n");
            break;
        }

        /* Display his info */
        if (blockType == CODE_START) /* First packet, with 10 bytes (general data) */
        {
            /* Get block size */
            blockSize = 10;

            /* Write info */
            #ifdef SHOW_OUTPUT
                printf("Start code found at offset %li, %u bytes\n", ftell(pssFile), blockSize);
            #endif

            /* Skip */
            fseek(pssFile, blockSize, SEEK_CUR);
        }
        else if (blockType == CODE_HEADER_START)
        {
            /* Get block size */
            freadBe(&blockSize, 2, 1, pssFile);

            /* Write info */
            #ifdef SHOW_OUTPUT
                printf("Start header code found at offset %li, %u bytes\n", ftell(pssFile), blockSize);
            #endif

            /* Skip */
            fseek(pssFile, blockSize, SEEK_CUR);
        }
        else if (blockType == CODE_VIDEO)
        {
            /* Get block size */
            freadBe(&blockSize, 2, 1, pssFile);

            /* Write info */
            #ifdef SHOW_OUTPUT
                printf("Video code found at offset %li, %u bytes\n", ftell(pssFile), blockSize);
            #endif

            /* Skip */
            fseek(pssFile, blockSize, SEEK_CUR);
        }
        else if (blockType == CODE_PADDING)
        {
            /* Get block size */
            freadBe(&blockSize, 2, 1, pssFile);

            /* Write info */
            #ifdef SHOW_OUTPUT
                printf("Padding code found at offset %li, %u bytes\n", ftell(pssFile), blockSize);
            #endif

            /* Skip */
            fseek(pssFile, blockSize, SEEK_CUR);
        }
        else if (blockType == CODE_PRIVATE_STREAM_1)
        {
            unsigned long int readVgsBytes;
            unsigned char copyBuffer[MAX_BLOCK_SIZE];

            unsigned char preAudioSize;
            unsigned short audioSize;

            /* Get block size */
            freadBe(&blockSize, 2, 1, pssFile);

            /* Write info */
            #ifdef SHOW_OUTPUT
                printf("Private stream (1) code found at offset %li, %u bytes\n", ftell(pssFile), blockSize);
            #endif

            /* Jump to data start */
            fseek(pssFile, 2, SEEK_CUR); // Jump 2 bytes
            preAudioSize = fgetc(pssFile); // Read a size (?)
            fseek(pssFile, preAudioSize, SEEK_CUR); // Jump that data
            fseek(pssFile, 4, SEEK_CUR); // Jump 4 bytes more

            /* Calculate definitive sizes */
            preAudioSize += 3;
            preAudioSize += 4;

            audioSize = blockSize - preAudioSize;

            if (mode == 'e') /* Extract VGS */
            {
                /* Read a block from the PSS */
                readVgsBytes = fread(copyBuffer, 1, audioSize, pssFile);

                /* Overwrite the PSS data */
                fwrite(copyBuffer, 1, readVgsBytes, vgsFile);

                /* Recalculate audio size */
                audioSize -= readVgsBytes;
                vgsCopiedSize += readVgsBytes;
            }
            else if (mode == 'i') /* Inject VGS */
            {
                /* Read a block from the VGS */
                readVgsBytes = fread(copyBuffer, 1, audioSize, vgsFile);

                /* Overwrite the PSS data */
                fwrite(copyBuffer, 1, readVgsBytes, pssFile);

                /* Recalculate audio size, and add it to the written byte count */
                audioSize -= readVgsBytes;
                vgsCopiedSize += readVgsBytes;
            }

            /* Skip the read of the data (if any) */
            fseek(pssFile, audioSize, SEEK_CUR);
        }
        else if (blockType == CODE_END)
        {
            /* Write info */
            #ifdef SHOW_OUTPUT
                printf("End code found at offset %li, %u bytes\n", ftell(pssFile), blockSize);
            #endif

            /* Break */
            break;
        }
        else
        {
            printf("ERROR: Unknown header code 0x%.8X at %li!\n", blockType, ftell(pssFile));
            StopProgram();
            exit(-1);
        }
    }

    /* Write extraction/injection report */
    if (mode == 'e')
    {
        printf("\nCopied %li bytes to the VGS file!\n", vgsCopiedSize);
    }
    else if (mode == 'i')
    {
        printf("\nCopied %li bytes of a total of %li bytes!\n", vgsCopiedSize, vgsFileSize);
        if (vgsCopiedSize < vgsFileSize)
        {
            printf("WARNING: Not all VGS data has been copied!\n");
        }
    }

    printf("\nProgram finished!\n");
    StopProgram();

    /* Close */
    fclose(pssFile);
    fclose(vgsFile);

    return 0;
}
